/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/
"use strict";


const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://gre/modules/Console.jsm");
Cu.import("resource://gre/modules/Services.jsm");

const anonymox = {};






var operationIndex = 0;

function startup(data, reason) {
	operationIndex += 1;


	data.webExtension.startup().then(api => {
		const {browser} = api;
		browser.runtime.onMessage.addListener(anonymox.core.websiteCommHandler);

		let prefObj = iteratePrefs();
		browser.runtime.onConnect.addListener((port) => {
			port.postMessage(prefObj);
		});

	});

	Cu.import("chrome://anonymox/content/modules/components.jsm", anonymox);

	Services.wm.addListener(windowListener);

	const windows = Services.wm.getEnumerator("navigator:browser");
	while (windows.hasMoreElements()) {
		const win = windows.getNext();
		initializeForWindow(win);
	}
}

function shutdown(data, reason) {
	operationIndex += 1;

	if (reason === APP_SHUTDOWN) {


		return;
	}

	Services.wm.removeListener(windowListener);

	anonymox.core.shutdownStartedHook();
	anonymox.core.uninstallHook();

	const windows = Services.wm.getEnumerator("navigator:browser");
	while (windows.hasMoreElements()) {
		const win = windows.getNext();
		cleanUpForWindow(win);
	}

	Services.strings.flushBundles();
}



function install(data, reason) {}
function uninstall(data, reason) {}


function initializeForWindow(win) {
	const info = new anonymox.Info("bootstrap::initializeForWindow");

	if (anonymox.state.offDueToFailure) {
		info.debug("Not initializing add-on for window: Off due to failure");
		return;
	}

	anonymox.domBuilder.buildOverlay(win.document);

	loadScripts(win, [
		"chrome://anonymox/content/modules/external/jquery-2.2.4.min.js",
		"chrome://anonymox/content/panel.js"
	]);

	win.anonymox.jQuery = win.jQuery.noConflict(true);




	if (!anonymox.state.isLoaded) {
		anonymox.core.startUp(win);
	}

	anonymox.events.onNewWindowOpened(win);

	function loadScripts(win, paths) {
		const subScriptLoader =
			Cc["@mozilla.org/moz/jssubscript-loader;1"]
				.getService(Ci.mozIJSSubScriptLoader);

		paths.forEach(function(path) {
			subScriptLoader.loadSubScript(
				path,
				win
			);
		});
	}
}

function cleanUpForWindow(win) {
	anonymox.domBuilder.clearAll(win.document);
}

function iteratePrefs() {
	let prefbranch = anonymox.prefs;
	let prefObj = {};

	var children = prefbranch.getChildList("", {});
	for (var i = 0; i < children.length; i++) {
		let prefName = children[i];
		let prefType = prefbranch.getPrefType(prefName);
		let prefVal;
		switch (prefType) {
			case prefbranch.PREF_BOOL:
				prefVal = prefbranch.getBoolPref(prefName);
				break;
			case prefbranch.PREF_INT:
				prefVal = prefbranch.getIntPref(prefName);
				break;
			case prefbranch.PREF_STRING:
				prefVal = prefbranch.getStringPref(prefName);
				if (prefName == "lastVersion" ||
					prefName == "profiles.list"
				) {
					prefVal = JSON.parse(prefVal);
				}
				break;
			default:
				break;
		}
		if (prefVal) {
			prefObj[prefName] = prefVal;
		}
	}
	return prefObj;
}


const windowListener = {
	onOpenWindow: function(xulWindow) {
		const domWindow =
			xulWindow
				.QueryInterface(Ci.nsIInterfaceRequestor)
				.getInterface(Ci.nsIDOMWindow);



		const thisOperationIndex = operationIndex;

		const onLoad = function() {
			domWindow.removeEventListener("load", onLoad);




			const windowType = domWindow.document.documentElement.getAttribute("windowtype");
			if (windowType !== "navigator:browser") {
				return;
			}













			if (thisOperationIndex !== operationIndex) {
				console.log("Anonymox", "Abort initialization of window");
				return;
			}

			initializeForWindow(domWindow);
		};

		domWindow.addEventListener("load", onLoad);
	},

	onCloseWindow: function(aWindow) {},
	onWindowTitleChange: function(aWindow, aTitle) {}
};
