/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");


var browserapi = {
	init : function() {
		domBuilder.buildStatusBar();
		browserapi.commonDialogObserver.register();
	},

	shutDown: function() {
		browserapi.stopProxy();
		browserapi.commonDialogObserver.unregister();
	},



	messageWindows : Array(),





	openURI : function(uri, level)
	{



		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var browserEnumerator = wm.getEnumerator("navigator:browser");

		var found = false;
		while (!found && browserEnumerator.hasMoreElements()) {
			var browserWin = browserEnumerator.getNext();
			var tabbrowser = browserWin.gBrowser;


			var numTabs = tabbrowser.browsers.length;
			for ( var index = 0; index < numTabs; index++) {
				var currentBrowser = tabbrowser.getBrowserAtIndex(index);
				if (uri == currentBrowser.currentURI.spec) {

					tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];


					browserWin.focus();

					found = true;
					break;
				}
			}
		}



		if (!found)
		{

			if (level == 1){

				if (Array_inArray(browserapi.messageWindows,uri)){
					browserapi.messageWindows[uri].close();
				}else
					browserapi.messageWindows[uri] = wm.getMostRecentWindow("navigator:browser")
					.open(uri, "anonymoX Message", "width=990,height=700,resizable=yes,scrollbars=yes,status=yes,location=yes")
					.focus();

			}else{

				var recentWindow = wm.getMostRecentWindow("navigator:browser");
				if (recentWindow) {

					recentWindow.delayedOpenTab(uri, null, null, null, null);
				} else {

					win.open(uri);
				}
			}
		}
	},

	closeOpenedURIs : function()
	{
		for ( var int = 0; int < browserapi.messageWindows.length; int++) {
			var win = browserapi.messageWindows[int];
			try{
				win.close();
			}catch(e){}
		}
	},



	isReferrerEnabled : function(){
		return config.allPrefs.getIntPref("network.http.sendRefererHeader") != 0;
	},

	setReferrer : function(enabled){
		config.allPrefs.setIntPref("network.http.sendRefererHeader", (enabled ? 2 : 0));
	},



	setUserAgent : function(value) {
		var info = new Info("setUserAgent");
		if (value != undefined)
			config.allPrefs.setCharPref("general.useragent.override", value);
		else
			config.allPrefs.clearUserPref("general.useragent.override");
		info.sendMsg(utils.msgType.usage, "features", "setUserAgent");
	},



	acceptCookies : function(accept, src){
		let info = new Info("browserapi.acceptCookies");

		try {
			config.allPrefs.setIntPref("network.cookie.cookieBehavior", accept);
			info.sendMsg(utils.msgType.usage, "features", "acceptCookies " + accept + "|" + src);
		}
		catch (e) {
			info.error(e);
		}
	},

	clearCookies : function(domain, src){
		var info = new Info("browserapi.clearCookies");
		info.debug(domain);
		var cookieMgr = Cc["@mozilla.org/cookiemanager;1"].getService(Ci.nsICookieManager);
		if (!domain)
			cookieMgr.removeAll();
		else
			for (var e = cookieMgr.enumerator; e.hasMoreElements();) {
				var cookie = e.getNext().QueryInterface(Components.interfaces.nsICookie);
				if (cookie.host.indexOf(domain) != -1){
					info.debug("delete " + cookie.host + ";" + cookie.name + "=" + cookie.value);
					cookieMgr.remove(cookie.host, cookie.name, cookie.path, false);
				}
			}
		info.sendMsg(utils.msgType.usage, "features", "clearCookies" + (domain?"_all":"") + "|" + src);
	},

	clearCookiesIdentChange : function (profile){
		var info = new Info("browserapi.clearCookiesIdentChange");


		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var win = wm.getMostRecentWindow("navigator:browser");
		if (!win) {
			new info.notice("No window found!");
			return;
		}

		info.verbose(profile.name + " (site=" + profile.site + "), " + profile.cookieAction);
		switch(profile.cookieAction){
			case 1:
				var domain = utils.getDomainPartsFromURI(win.anonymoxdata.uri);
				browserapi.clearCookies(domain[0], "identChange");
				break;
			case 2:
				browserapi.clearCookies(undefined, "identChange");
				break;
		}
	},



	checkForEnabledProxy : function(callback){

		let proxyProtocolService = Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService);
		let ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);

		proxyProtocolService.asyncResolve(ioService.newURI("http://www.anonymox.net/", null, null), 0, {
			onProxyAvailable : function(aRequest, channelOrUri, aProxyInfo, aStatus){





				callback (aProxyInfo && aProxyInfo.type != "direct");
			}
		});
	},

	startProxy : function() {
		browserapi.proxyService.register();
		browserapi.httpRequestObserver.register();
	},

	stopProxy : function() {
		browserapi.proxyService.unregister();
		browserapi.httpRequestObserver.unregister();
	},


	setProxyAuth : function() {
		var authMgr = Components.classes["@mozilla.org/network/http-auth-manager;1"].getService(Components.interfaces.nsIHttpAuthManager);

		core.gatewayListCache.forEach(
			function(gw) {

				authMgr.setAuthIdentity("http", gw.tls ? gw.tlsHostname : gw.ip, gw.port, "digest", "anonymox.net", "", "", config.user, config.pass);
			}
		);
	},


	proxyService : {
		proxyProtocolService : Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService),

		applyFilter : function(ps, uri, preassigedProxy) {
			var info = new Info("proxyService.applyFilter");
			try {
				if (state.isReady) {
					var profile = profilemgr.getProfileForURI(uri.asciiSpec);
					events.onProfileIsBeingUsed(profile);
					if (profile.proxyServiceInfo) {
						var pi = profile.proxyServiceInfo;

						return browserapi.proxyService.proxyProtocolService.newProxyInfo(pi.tls ? "https" : "http", pi.tls ? pi.tlsHostname : pi.ip, pi.port, 0, null, null);
					} else {

						return preassigedProxy;
					}
				} else {
					info.debug(uri.asciiSpec + " -> RC: state.isReady==false");
					return preassigedProxy;
				}
			} catch (e) {
				info.error(e);
			}
		},

		register : function() {
			try {
				Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService).registerFilter(this, 0);
			} catch (e) {
				new Info("proxyService.register").error(e);
			}
		},

		unregister : function() {
			try {
				Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService).unregisterFilter(this);
			} catch (e) {
				new Info("proxyService.unregister").error(e);
			}
		}
	},


	httpRequestObserver : {
		info : new Info("httpRequestObserver"),

		observe : function(subject, topic, data) {
			try {
				var httpChannel = subject.QueryInterface(Ci.nsIHttpChannel);


				if (httpChannel.URI.asciiSpec.match('^https?://([^/]+\.)?anonymox.net/')){
					httpChannel.setRequestHeader("X-AnonymoX-Auth", "uid:" + config.user, false);
					httpChannel.setRequestHeader("X-AnonymoX-Capabilities", config.capabilities, false);
				}
			} catch (e) {
				this.info.error(e);
			}
		},
		register : function() {
			try {
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.addObserver(this, "http-on-modify-request", false);
			} catch (e) {
				this.info.error(e);
			}
		},
		unregister : function() {
			try {
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.removeObserver(this, "http-on-modify-request");
			} catch (e) {

			}
		}
	},


	progressListener: {
		QueryInterface : function(aIID) {
			if (aIID.equals(Ci.nsIWebProgressListener)
					|| aIID.equals(Ci.nsISupportsWeakReference)
					|| aIID.equals(Ci.nsISupports))
				return this;
			throw Components.results.NS_NOINTERFACE;
		},

		onLocationChange: function(aWebProgress, aRequest, aURI) {
			const info = new Info("browserapi.progressListener.onLocationChange");
			info.verbose("onLocationChange " + aURI.asciiSpec);

			try {
				events.onWindowAddressUpdate();
			} catch(e) {
				info.error(e);
			}
		}
	},

	commonDialogObserver: {
		observe: function(win, aTopic, aData) {
			if (aTopic != "domwindowopened") {
				return;
			}

			win.addEventListener("load", onLoadListener);
			function onLoadListener() {
				win.removeEventListener("load", onLoadListener);

				if (win.document.URL === "chrome://global/content/commonDialog.xul") {
					events.onCommonDialogOpened(win);
				}
			}
		},

		register: function() {
			const windowWatcher =
				Cc['@mozilla.org/embedcomp/window-watcher;1']
					.getService(Ci.nsIWindowWatcher);

			windowWatcher.registerNotification(browserapi.commonDialogObserver);
		},

		unregister: function() {
			const windowWatcher =
				Cc['@mozilla.org/embedcomp/window-watcher;1']
					.getService(Ci.nsIWindowWatcher);

			windowWatcher.unregisterNotification(browserapi.commonDialogObserver);
		}
	}
};
