/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";


var config = {

	defaultPrefs: [
		["extensions.anonymox.firstrun", true],


		["extensions.anonymox.user", ""],
		["extensions.anonymox.pass", ""],





		["extensions.anonymox.lastPremium"       , false],
		["extensions.anonymox.displayInStatusBar", true ],
		["extensions.anonymox.displayGateway"    , false],
		["extensions.anonymox.displayIP"         , false],
		["extensions.anonymox.displayFlag"       , true ],
		["extensions.anonymox.debugout"          , false],
		["extensions.anonymox.iconCreated"       , false],

		["extensions.anonymox.showNotifications.identChangeCookieAction"        , true ],
		["extensions.anonymox.showNotifications.autoGatewayChanged"             , false],
		["extensions.anonymox.showNotifications.panelNotForThisSite"            , true ],
		["extensions.anonymox.showNotifications.configuredProxyDetectedOverride", true ],
		["extensions.anonymox.showNotifications.netErrorWilltryAgain"           , true ],
		["extensions.anonymox.showNotifications.newIdentityRotateReset"         , true ],
		["extensions.anonymox.showNotifications.newProfileCreated"              , true ],
		["extensions.anonymox.showNotifications.disableCookiesDeleteHint"       , true ],

		["extensions.anonymox.usagecollection"  , 0    ],
		["extensions.anonymox.displayProfileBar", false],

		["extensions.anonymox.profiles.list", "[{\"name\":\"youtube.com\",\"site\":\"youtube.com\",\"uriFilterType\":0,\"gatewayFilter\":{\"enabled\":true,\"country\":\"us\",\"attributes\":{\"stealth\":true}}},{\"name\":\"default\",\"site\":\"*\",\"uriFilterType\":1,\"uriFilter\":\".*\",\"gatewayFilter\":{\"enabled\":true,\"attributes\":{}}}]"]
	],

	init : function() {
		config.setDefaultPrefs();

		config.version = "3.3.1";
		config.capabilities = "oneclickactivate";



		config.masterURI = "https://master.anonymox.net/firefox";




		config.networkInfoURI = "http://nwi.anonymox.net/";
		config.networkInfoSecureURI = "https://nwi.anonymox.net/";
		config.homepageBaseUri = "http://www.anonymox.net/" + localize("lang_id") + "/";
		config.supportFeedbackPageUri = config.homepageBaseUri + "support-feedback";
		config.helpErrorPageUri = config.homepageBaseUri + "help-error";
		config.profilesWarnUri = config.homepageBaseUri + "profiles-warning";
		config.premiumTrialUri = "http://clientinfo.any.mx/" + localize("lang_id") + "/premiumtrial";
		config.welcomeSiteUri = config.homepageBaseUri + "gettingstarted";
		config.selfCheckFailedUri = config.homepageBaseUri + "self-check-failed";
		config.identListHelpUri = config.homepageBaseUri + "identlist-help";
		config.premiumInfoURI = config.homepageBaseUri + "premiuminfo?src=";
		config.disableAdsURI = config.homepageBaseUri + "disable-ads";
		config.usageCollectionInfoUri = config.homepageBaseUri + "usagecollectioninfo";

		config.extensionID = "client@anonymox.net";

		config.languageID = localize("lang_id");

		config.installPath = undefined;

		config.attributes = ["stealth", "fast", "premium"];
		config.premiumAttributes = ["fast", "premium"];

		config.osname = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime).OS;
		config.browserVersion = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo).version;

		config.httpTimeoutSec = 15;
		config.refreshInfoIntervalMin = 15;
		config.netRetryWaitTimeSec = 30;
		config.usageCollectionIntervalHrs = 24 * 3;
		config.selfCheckIntervalMin = 60;
		config.externalInfoAutoReloadMin = 15;
		config.siteIsConsideredBusyIntervalMS = 60 * 1000;
		config.adBoxNoAutoCloseIntervalSec = 2;
		config.changeGWThresholdScore = 1;

		Cu.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.getAddonByID(config.extensionID, function (addon) {
			config.installPath =  addon.getResourceURI("").QueryInterface(Ci.nsIFileURL).file.path;
		});

		config.prefListener = new config.PrefListener("extensions.anonymox.",
			function (branch, name) {
				try {
					switch (name) {
						case "user":
							config.user = config.prefs.getCharPref("user");
							break;
						case "pass":
							config.pass = config.prefs.getCharPref("pass");
							break;
						case "displayIP":
						case "displayFlag":
						case "displayGateway":
						case "displayInStatusBar":
							events.onUiConfigUpdated();
							break;
						case "debugout":
							config.debugout = config.prefs.getBoolPref("debugout");
					}
				} catch(e) {
					new Info("config.prefListener").error(e);
				}
			}
		);
		config.prefListener.register();
	},




	setDefaultPrefs: function() {
		Components.utils.import("resource://gre/modules/Services.jsm");
		var branch = Services.prefs.getDefaultBranch(null);

		config.defaultPrefs.forEach(function(defaultPref) {
			let name  = defaultPref[0];
			let value = defaultPref[1];

			switch (typeof value) {
				case "string":
					branch.setCharPref(name, value);
					break;

				case "number":
					branch.setIntPref(name, value);
					break;

				case "boolean":
					branch.setBoolPref(name, value);
					break;
			}
		});
	},

	PrefListener : function (branchName, func) {
		var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
		var branch = prefService.getBranch(branchName);
		branch.QueryInterface(Ci.nsIPrefBranch);

		this.register = function() {
			branch.addObserver("", this, false);
			branch.getChildList("", { }).forEach(function (name) { func(branch, name); });
		};

		this.unregister = function unregister() {
			if (branch)
				branch.removeObserver("", this);
		};

		this.observe = function(subject, topic, data) {
			if (topic == "nsPref:changed")
				func(branch, data);
		};
	},

	resetAllPrefs : function(includeAnonID) {




		config.resetPref("displayInStatusBar");
		config.resetPref("displayGateway");
		config.resetPref("displayIP");
		config.resetPref("displayFlag");
		config.resetPref("debugout");
		config.resetPref("profiles.list");
		config.resetPref("usagecollection");
		config.resetPref("iconCreated");
		config.resetPref("displayProfileBar");
		config.resetPref("lastPremium");
		config.resetPref("usagecollection");

		config.resetPref("showNotifications.identChangeCookieAction");
		config.resetPref("showNotifications.autoGatewayChanged");
		config.resetPref("showNotifications.panelNotForThisSite");
		config.resetPref("showNotifications.configuredProxyDetectedOverride");
		config.resetPref("showNotifications.netErrorWilltryAgain");
		config.resetPref("showNotifications.newIdentityRotateReset");
		config.resetPref("showNotifications.newProfileCreated");
		config.resetPref("showNotifications.disableCookiesDeleteHint");

		if (includeAnonID) {
			config.resetPref("firstrun");
			config.resetPref("user");
			config.resetPref("pass");
		}
	},

	saveOriginalGlobalPrefsIfNotDone : function() {

		if (!prefs.prefHasUserValue("original.network.http.sendRefererHeader")) {
			if (config.allPrefs.prefHasUserValue("general.useragent.override")) {
				prefs.setCharPref("original.general.useragent.override", config.allPrefs.getCharPref("general.useragent.override"));
			} else {
				config.resetPref("original.general.useragent.override");
			}
			prefs.setIntPref("original.network.http.sendRefererHeader", config.allPrefs.getIntPref("network.http.sendRefererHeader"));
			prefs.setIntPref("original.network.cookie.cookieBehavior", config.allPrefs.getIntPref("network.cookie.cookieBehavior"));
		}
	},

	restoreGlobalPrefs : function() {
		if (prefs.prefHasUserValue("original.general.useragent.override")) {
			config.allPrefs.setCharPref("general.useragent.override", prefs.getCharPref("original.general.useragent.override"));
		} else {
			config.resetPrefGlobal("general.useragent.override");
		}
		if (prefs.getPrefType('original.network.http.sendRefererHeader')) {
			config.allPrefs.setIntPref("network.http.sendRefererHeader", prefs.getIntPref("original.network.http.sendRefererHeader"));
		}
		if (prefs.getPrefType('original.network.cookie.cookieBehavior')) {
			config.allPrefs.setIntPref("network.cookie.cookieBehavior", prefs.getIntPref("original.network.cookie.cookieBehavior"));
		}

		config.resetPref("original.network.http.sendRefererHeader");
		config.resetPref("original.general.useragent.override");
		config.resetPref("original.network.cookie.cookieBehavior");
	},

	resetPref: function(name) {
		config.resetPrefGlobal("extensions.anonymox." + name);
	},

	resetPrefGlobal: function(name) {
		let info = new Info("config.resetPref");
		try {

			Services.prefs.clearUserPref(name);
		} catch(e) {
			info.notice(name + " : " +  e);
		}
	}
};

var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
config.prefs = prefService.getBranch("extensions.anonymox.");
config.allPrefs = prefService.getBranch("");


var prefs = config.prefs;
