/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";


var core = {
	startUp : function(win) {
		var info = new Info("core.startUp");
		try {
			state.isLoaded = true;

			config.init();
			utils.init(win);
			info.debug(utils.versionInfo);
			state.init();
			communication.init();
			profilemgr.init();
			browserapi.init();

			core.upgradeSettingsIfRequired();

			core.switchOn();
		} catch (e) {
			info.error(e);
		}
	},

	switchOn : function() {
		var info = new Info("core.switchOn");
		try {
			state.setSystemState(state.systemStates.loading);

			state.isStarting = true;

			config.saveOriginalGlobalPrefsIfNotDone();

			if (prefs.getBoolPref("firstrun")) {

				core.firstRun();
			} else {
				browserapi.checkForEnabledProxy(function(otherIsEnabled) {
					if (otherIsEnabled)
						events.onNotifyUser(0,
								"configuredProxyDetectedOverride");
				});

				core.refreshInfo(function() {

					try {
						var info = new Info("switchOn(refreshedInfo)");
						info.debug("");




						if (!state.isStarting) {
							info.debug("Aborting callback. Add-on not starting any more.");
							return;
						}

						browserapi.startProxy();

						state.isReady = true;

						core.getAllExternalInfoIfNeeded();

						state.setSystemState(state.systemStates.ready);

						state.isStarting = false;





						if (prefs.getIntPref("usagecollection") == 1){

							setTimeout(function(){
								var pl = "";
								for ( var i = 0; i < profilemgr.list.length; i++) {
									if (pl!="")
										pl+="|";
									var profile = profilemgr.list[i];
									if (!profile.isInternal){
										pl+= profile.site + ";";
										pl+= profile.gatewayFilter.enabled + ";" + profilemgr.getCountryName(profile.gatewayFilter.country) + ";";
									}
								}
								communication.csclient.Log(utils.msgType.usage, "profiles", pl, config.user, undefined, function(){});
							}, 200);
						}


						if (core.showWelcomeSiteOnActivationTMP) {
							delete core.showWelcomeSiteOnActivationTMP;
							browserapi.openURI(config.welcomeSiteUri, 0);
						}
					} catch (e) {
						info.error(e);
					}
				});
			}
		} catch (e) {
			info.error(e);
		}
	},

	uninstallHook : function() {
		core.switchOff();
		config.restoreGlobalPrefs();
	},

	shutdownStartedHook : function(){
		state.isReady = false;
		browserapi.closeOpenedURIs();
	},

	upgradeSettingsIfRequired : function() {
		var info = new Info("core.upgradeSettingsIfRequired");
		try {
			var lvString = prefs.getCharPref("lastVersion");
		} catch(e) {}

		let current = JSON.stringify({
			addon : config.version,
			browser : config.browserVersion
		});

		if (!lvString) {
			info.debug("NOT upgrade from 0.9.3 or lower...");
		} else {
			let lastVersion;
			try {
				lastVersion = JSON.parse(lvString);
			} catch(e) {
				lastVersion = {
					addon : lvString,
					browser : config.browserVersion
				}
			}
			if (lastVersion.addon != config.version
					|| lastVersion.browser != config.browserVersion) {
				info.debug("update addon: " + lastVersion.addon + " to "
						+ config.version + "; browser: " + lastVersion.browser
						+ " to " + config.browserVersion);

				if (lastVersion.addon != config.version) {
					config.resetPref("addonBar");
				}

				var versionChecker = Cc["@mozilla.org/xpcom/version-comparator;1"]
						.getService(Ci.nsIVersionComparator);

				const oldBrowserBelow28 = versionChecker.compare(
						lastVersion.browser, "28") < 0;
				const newBrowserVersionAtLeast28 = versionChecker.compare(
						config.browserVersion, "28") >= 0;



				if (oldBrowserBelow28 && newBrowserVersionAtLeast28) {
					if (!prefs.getBoolPref("displayInStatusBar")) {
						prefs.setBoolPref("displayGateway", false);
						prefs.setBoolPref("displayIP", false);
						prefs.setBoolPref("displayFlag", false);
						events.onUiUpdateRequired();
					}
				}
			}
		}
		prefs.setCharPref("lastVersion", current);


		config.resetPref("profiles.overrideByDefault");
	},



	cleanState : function() {
		state.resetAll();
		delete core.gatewayListCache;
	},

	switchOff : function() {
		var info = new Info("core.switchOff");
		info.notice("");

		state.isStarting = false;
		state.isReady = false;
		state.isLoaded = false;

		state.setSystemState(state.systemStates.disabled);

		events.onSwitchOff();

		browserapi.shutDown();

		core.cleanState();


		if (core.refreshTimer) {
			core.refreshTimer.cancel();
			core.refreshTimer = null;
		}

	},



	refreshInfo : function(continueFunc, intervalInvoked) {
		try {
			var info = new Info("core.refreshInfo");
			var retryNum = 0;

			info.debug(utils.displayFuncOrUndefined(continueFunc) + ", " + intervalInvoked);

			if (intervalInvoked && !state.isReady)
				return;

			let startCall;

			var successFunc = function(result) {
				try {
					events.onRemoveNotificationForAllWindows("netErrorWilltryAgain");


					if (intervalInvoked && !state.isReady)
						return;

					info.verbose("Received gateways:");
					for (var i = 0; i < result.gateways.length; i++) {
						var gw = result.gateways[i];
						info.verbose(gw.id + " " + gw.ip + ":" + gw.port + " " + gw.country + " usage " + gw.usageIndex);
					}

					state.localExternalInfo = result.externalInfo;
					state.adBoxInfo = result.adBoxInfo;

					if (state.displayStatusURI != result.statusUri) {
						state.displayStatusURI = result.statusUri;

						setTimeout(
								function() {

									var win = Cc["@mozilla.org/appshell/window-mediator;1"]
											.getService(Ci.nsIWindowMediator)
											.getMostRecentWindow(
													"navigator:browser");
									win.anonymox.panel.open("networkStatus");
								}, 2000);
					}

					events.onRefreshInfo(result);

					core.updatePremiumStatus(result.premium, true);
					core.showPremTrial = result.showPremiumTrialOffer;

					if (!core.gatewayListCache)
						core.gatewayListCache = new Array();

					info.verbose("gwlistcache: " + info.serializeArray(core.gatewayListCache, function(inp){return inp.id;}));
					info.verbose("result.gateways: " + info.serializeArray(result.gateways, function(inp){return inp.id;}));




					for ( var i = core.gatewayListCache.length - 1; i >= 0; i--) {
						var oldgw = core.gatewayListCache[i];

						var foundmatch = false;

						for ( var j = result.gateways.length - 1; j >=0; j--) {
							var newgw = result.gateways[j];

							if(newgw.id == oldgw.id){

								var propchanged = false;
								for (var property in newgw) {
									var prop_old = JSON.stringify(oldgw[property]);
									var prop_new = JSON.stringify(newgw[property]);
									if (prop_old != prop_new){
										info.verbose(newgw.id + " Property changed: " + property + " old " + prop_old + " new " + prop_new);
										oldgw[property] = newgw[property];
										propchanged = true;
									}
								}

								foundmatch = true;
								break;
							}
						}
						if (!foundmatch) {
							Array_remove(core.gatewayListCache, i);
						}
					}


					for (var j = result.gateways.length - 1; j >=0; j--) {
						var newgw = result.gateways[j];
						var foundmatch = false;

						for (var i = core.gatewayListCache.length - 1; i >= 0; i--) {
							var oldgw = core.gatewayListCache[i];
							if (newgw.id == oldgw.id) {
								foundmatch = true;
								break;
							}
						}

						if (!foundmatch) {
							core.gatewayListCache.push(newgw);
						}
					}

					browserapi.setProxyAuth();

					profilemgr.assignGWForAll();


					if (result.showURL != null) {
						browserapi.openURI(result.showURL, 1);
					}

					if (result.forceUpgradeVersion) {
						events.onAddOnUpdateRequested(result.forceUpgradeVersion);
					}

					core.refreshInfo_nextIntervalTO = setTimeout(function() {
						core.refreshInfo(null, true);
					}, 1000 * 60 * config.refreshInfoIntervalMin);


					if (continueFunc) continueFunc.call();
				} catch (e) {

					if (e.toString().indexOf("attempt to run compile-and-go script on a cleared scope") != -1){
						info.debug("catched 'attempt to run compile-and-go script on a cleared scope'");
						info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'attempt to run compile-and-go script on a cleared scope'");
						core.refreshInfo(continueFunc, intervalInvoked);
						return;
					}

					info.error(e, "E005: Not able to process clientInfo.");
				}
			};

			var errorFunc = function(xhr, status, error) {
				if (error instanceof GenericException2) {
					if (error.uri)
						browserapi.openURI(error.uri, 1);
					else
						events.onNotifyUserModal("systemFatalError", error.xmessage);
					core.switchOff();
				} else {
					info.notice("Net or Server error. Will try again in " + config.netRetryWaitTimeSec + " sec. Message: \"" + JSON.stringify(xhr) + "\"");
					retryNum++;
					if (retryNum == 5) events.onNotifyUser(0, "netErrorWilltryAgain");
					core.refreshInfo_retryTO = setTimeout(startCall, config.netRetryWaitTimeSec * 1000);
				}
			};

			startCall = function() {
				communication.csclient.Info7(config.user, config.pass, config.version, config.languageID, config.osname , successFunc).error(errorFunc);
			};

			startCall();
		} catch (e) {

			if (e.toString().indexOf("attempt to run compile-and-go script on a cleared scope") != -1){
				info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'attempt to run compile-and-go script on a cleared scope'");
				core.refreshInfo(continueFunc, intervalInvoked);
				return;
			}

			if (e.toString().indexOf("second is undefined") != -1){
				info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'second is undefined'");
				core.refreshInfo(continueFunc, intervalInvoked);
				return;
			}

			if (e.toString().indexOf("0x80070057") != -1) {
				core.tmpTO = setTimeout(
					function() {
						core.refreshInfo(continueFunc, intervalInvoked);
					},
					60 * 1000);
				return;
			}


			if (!intervalInvoked || (intervalInvoked && e.lineNumber != 210))
				info.error(e, "E006: Not able to get clientInfo.");
		}
	},

	updatePremiumStatus : function(active, infoIsFresh) {
		var info = new Info("updatePremiumStatus");
		info.verbose(active);

		if (core.isPremiumAccount != active) {
			core.isPremiumAccount = active;

			if ((core.isPremiumAccount == true) != prefs
					.getBoolPref("lastPremium")) {
				prefs.setBoolPref("lastPremium", core.isPremiumAccount);
				if (!infoIsFresh)
					core.refreshInfo(function() {
						core.premiumStatusChanged();
					});
				else
					core.premiumStatusChanged();
			}
		}
	},

	premiumStatusChanged : function() {
		var info = new Info("premiumStatusChanged");

		info.notice("premium account is now "
				+ (core.isPremiumAccount ? "enabled" : "disabled"));
		events.onNotifyUser(1, "prem"
				+ (core.isPremiumAccount ? "_enabled" : "_disabled"));

		if (core.isPremiumAccount) {
			events.onPremiumActivated();
		} else {


			for (var i = 0; i < profilemgr.list.length; i++) {
				var pro = profilemgr.list[i];

				config.premiumAttributes.forEach(function(premiumAttrib) {
					pro.gatewayFilter.attributes[premiumAttrib] = undefined;
				});
			}

			events.onPremiumDeactivated();
		}
	},


	firstRun : function() {
		var info = new Info("core.firstRun");
		info.debug("");

		var retryNum = 0;

		try {
			let startCall;

			var successFunc = function(result) {
				config.user = result.user;
				config.pass = result.passwordPlain;
				prefs.setCharPref("user", result.user);
				prefs.setCharPref("pass", result.passwordPlain);

				prefs.setBoolPref("firstrun", false);
				events
						.onRemoveNotificationForAllWindows("netErrorWilltryAgain");

				core.showWelcomeSiteOnActivationTMP = true;
				core.switchOn();
			};

			var errorFunc = function(xhr, status, error) {
				if (error instanceof GenericException2) {
					if (error.uri)
						browserapi.openURI(error.uri, 1);
					else
						events.onNotifyUserModal("systemFatalError", error.xmessage);
					core.switchOff();
				} else {
					info.notice("Net or Server error. Will try again in " + config.netRetryWaitTimeSec + " sec. Message: \"" + JSON.stringify(xhr) + "\"");
					retryNum++;
					if (retryNum == 5) events.onNotifyUser(0, "netErrorWilltryAgain");
					core.refreshInfo_retryTO = setTimeout(startCall, config.netRetryWaitTimeSec * 1000);
				}
			};

			startCall = function() {
				communication.csclient.GetAccount2(2, successFunc).error(
						errorFunc);
			};

			startCall();

		} catch (e) {
			info.error(e);
		}
	},

	websiteCommHandler : function(message) {
		var info = new Info("websiteCommHandler");

		switch (message.cmd) {
		case "activatePremium":
			core.activatePremium(message.param1);
			break;
		}
	},


	activatePremium : function(code,  continueFunc) {
		var info = new Info("activatePremium");

		var successFunc = function(result) {
			core.updatePremiumStatus(true);
			info.debug("activated " + code);

			if (!state.isReady)
				core.switchOn();
			if (continueFunc)
				continueFunc();
		};

		var errorFunc = function(xhr, status, error) {
			if (error instanceof GenericException2) {
				if (error.uri)
					browserapi.openURI(error.uri, 1);
				else
					events.onNotifyUserModal("couldNotActivatePremium",
							error.xmessage);
			} else {
				info.debug(JSON.stringify(xhr));
				events.onNotifyUserModal("couldNotActivatePremium", "Please try again in a few minutes. Network or server error.");
			}

			if (continueFunc) continueFunc();
		};

		communication.csclient.ActivatePremium(code, config.user, config.pass, config.languageID, successFunc).error(errorFunc);
	},

	getBrowserFromChannel : function(aChannel) {
		var notificationCallbacks = aChannel.notificationCallbacks ? aChannel.notificationCallbacks : aChannel.loadGroup.notificationCallbacks;

		if (!notificationCallbacks) return null;

		var domWin = notificationCallbacks.getInterface(Ci.nsIDOMWindow);
		return domWin.gBrowser.getBrowserForDocument(domWin.top.document);

	},




	setGWstatus : function(gw, newStatus) {
		var info = new Info("core.setGWstatus");

		if (newStatus) {
			if (!gw.isOnline) {
				var formerStatus = gw.isOnline;
				info.verbose(gw.id + " is now online");
				if (gw == profilemgr.nullGateway) {
					info.debug("Assuming global online mode from now");
				}
				gw.isOnline = true;
				if (formerStatus == false) {
					profilemgr.assignGWForAll();
				}
			}
		} else {
			let setGWstatus_helper_setOff = function(gw) {
				if (gw.isOnline == undefined || gw.isOnline == true) {
					info.debug(gw.id + " failure. Temporary disabling this gateway(until next selfCheck).");
					gw.isOnline = false;
					profilemgr.assignGWForAll();
				}
			};

			if (gw == profilemgr.nullGateway) {
				if (gw.isOnline == undefined || gw.isOnline == true) {
					info.notice("Assuming global offline mode from now");
				}
				setGWstatus_helper_setOff(gw);
			} else {
				if (profilemgr.nullGateway.isOnline) {
					setGWstatus_helper_setOff(gw);
					info.sendMsg(utils.msgType.notice, "selfcheckfailed", gw.id);
				} else {
					info.debug("not disabling failed " + gw.id + " because we are in offline mode");
				}
			}
		}
	},

	selfCheckGW : function(gateway, useForMeasure) {
		if (config.disableSelfCheck)
			return;
		var info = new Info("core.selfCheckGW " + gateway.id);

		if (!state.isReady)
			return;

		try {
			if (!gateway.checkIsRunning) {
				info.verbose("");
				gateway.checkIsRunning = true;
				var hh = new communication.HttpHandler();
				var startTime = new Date().getTime();

				var successFunc = function(text) {
					if (!state.isReady)
						return;

					gateway.checkIsRunning = false;

					var lat = new Date().getTime() - startTime;

					info.verbose("success, " + lat + " ms");

					core.setGWstatus(gateway, true);
				};

				var failedFunc = function(a,b){
					if(!state.isReady) return;
					info.verbose("failed");
					gateway.checkIsRunning = false;
					core.setGWstatus(gateway, false);
				};

				var filter = (gateway != profilemgr.nullGateway ? /^OK$/ : /.*/);


				hh.doRequest(config.networkInfoURI + "selfcheck?gw=" + gateway.id, gateway, successFunc, failedFunc, filter);
			} else {
				info.verbose("Already being self-checked. skip");
			}
		} catch(e) {
			info.error(e);
		}
	},

	resetGatewayStatus : function() {
		var info = new Info("core.resetGatewayStatus");
		try {
			core.gatewayListCache.forEach(function(gw) {
				delete gw.isOnline;
			});
			profilemgr.assignGWForAll();
		} catch (e) {
			info.error(e);
		}
	},




	getAllExternalInfoIfNeeded : function(autorepeat) {
		var info = new Info("core.getAllExternalInfoIfNeeded");
		info.verbose("");

		for (var i = 0; i < profilemgr.list.length; i++) {
			var profile = profilemgr.list[i];
			core.getExternalInfo(profile.gateway);
		}

		if (autorepeat) {
			core.getAllExternalInfoIfNeededAutoTO = setTimeout(function() {
				core.getAllExternalInfoIfNeeded(true);
			}, config.externalInfoAutoReloadMin * 60 * 1000);
		}
	},

	getExternalInfoIfNeeded : function(gateway) {
		var info = new Info("getExternalInfoIfNeeded");
		if (prefs.getBoolPref("displayIP") || prefs.getBoolPref("displayFlag")) {
			info.verbose("");
			core.getExternalInfo(gateway);
		} else {
			info.verbose("exit, not needed");
		}
	},

	getExternalInfo : function(gateway, ani) {
		var info = new Info("getExternalInfo");
		info.verbose(gateway.id);

		if (!gateway.externalInfo) {
			gateway.externalInfo = {};
		}

		if (!state.isReady) {
			info.debug("core is not ready -> abort");
			return;
		}

		if (gateway.externalInfo.status == 1) {
			info.verbose("already being processed -> abort");
			events.onExternalInfoUpdated(gateway, false);
			return;
		}

		gateway.externalInfo.status = 1;
		events.onExternalInfoUpdated(gateway, false);

		var failedFunc = function(stat, add) {
			if (!state.isReady)
				return;
			gateway.externalInfo.status = -1;
			events.onExternalInfoUpdated(gateway, false);
			if (!core.sentExtInfoError) {
				info.sendMsg(utils.msgType.notice, "refreshExternalInfo",
						"failed|" + gateway.id + "|" + stat + "|" + add);
				core.sentExtInfoError = true;
			}
		};

		var successFunc = function(match) {
			if (!state.isReady)
				return;
			gateway.externalInfo.status = 2;
			gateway.externalInfo.ip = match[1];
			gateway.externalInfo.country = match[2];
			info.verbose("successful-> " + gateway.externalInfo.ip + ", "
					+ gateway.externalInfo.country);
			events.onExternalInfoUpdated(gateway, ani);
		};

		var hh = new communication.HttpHandler();


		hh.doRequest(config.networkInfoURI + "externalinfo?gw=" + gateway.id,
				gateway, successFunc, failedFunc, "^([^ ]{1,15}) (.{2})$");
	},

	domainCache : {},

	resolveDomain : function(domain, success) {
		var info = new Info("resolveDomain");

		let cache = core.domainCache;
		if (cache.hasOwnProperty(domain)) {
			success(cache[domain]);
			return;
		}

		var successFunc = function(response) {
			cache[domain] = response;
			success(response);
		};

		var failedFunc = function(stat, add) {

		};

		var hh = new communication.HttpHandler();
		hh.doRequest(config.networkInfoSecureURI + "remoteaddr?name=" + domain,
				null, successFunc, failedFunc, ".*");
	}
};
