/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/
"use strict";


const {
	BUTTON, HBOX, IFRAME, IMAGE, KEY, KEYSET, LABEL, PANEL, POPUPSET,
	TOOLBARITEM, TOOLBARBUTTON, VBOX, TOOLTIP, BOX, MENULIST, MENUPOPUP, SPACER,
	CHECKBOX, RICHLISTBOX, MENUITEM, LISTBOX, LISTHEAD, LISTHEADER, LISTCOLS,
	LISTCOL, MENU, MENUSEPARATOR,

	SECTION, DIV, MAIN, SPAN, H2
} = Xul;

Cu.import("resource:///modules/CustomizableUI.jsm");


var domBuilder = {


	buildStatusBar: function() {
		let info = new Info("overlay.buildStatusBar");

		CustomizableUI.createWidget({
			id : "anonymox-view-button",
			type: "custom",
			defaultArea : CustomizableUI.AREA_NAVBAR,
			onBuild : function(doc) {
				let node =
					TOOLBARITEM(
						{
							"id"       : "anonymox-view-button",
							"removable": "true",
							"label"    : "AnonymoX",
							"title"    : "AnonymoX"
						},
						TOOLBARBUTTON({
							"id"     : "anonymox-view-icon",
							"tooltip": "anonymox-infopanel",
							"class"  : "toolbarbutton-1",
							"label"  : "AnonymoX",
							"onclick": "anonymox.events.onOpenPanel(window, 'viewButton', event)"
						}),
						HBOX({ "id": "anonymox-statusinfo", "align": "center" },
							LABEL({
								"id"     : "anonymox-statusLabel",
								"style"  : "margin-left: 0px; padding-left: 2px;",
								"onclick": "anonymox.events.onOpenPanel(window, 'viewButton', event)"
							}),
							IMAGE({
								"id"     : "anonymox-externalFlag",
								"src"    : "chrome://",
								"onclick": "anonymox.events.onExternalInfoRefreshRequested(window)",
							}),
							LABEL({
								"id"     : "anonymox-externalIp",
								"onclick": "anonymox.events.onExternalInfoRefreshRequested(window)"
							})
						)
					);



				CustomizableUI.addListener(domBuilder.customizableUIListener);

				return node.build(doc);
			},
			onViewShowing : function() {
			},
			onCommand : function() {
			},
			onClick : function() {
			}
		});
	},



	buildOverlay: function(doc) {
		let nodes = [
			POPUPSET(

				PANEL(
					{
						"id"         : "anonymox-adbox",
						"noautohide" : "true",
						"level"      : "parent",
						"noautofocus": "true"
					},
					HBOX(
						IFRAME({
							"id"  : "anonymox-adbox-frame",
							"type": "content"
						}),
						VBOX({ "style": "text-align: right;"},
							BUTTON({
								"label"    : localize("adBox.close"),
								"oncommand": "anonymox.events.onCloseAdBoxClicked()",
								"style"    : "margin-bottom:0px;margin-top:0px;"
							}),
							BUTTON({
								"label"    : localize("adBox.disable"),
								"oncommand": "anonymox.events.onDisableAdBoxClicked()",
								"style"    : "margin-bottom:0px;margin-top:0px;"
							}),
							LABEL({
								"value": localize("adBox.marking"),
								"style": "font-size:8px;"
							})
						)
					)
				),


				TOOLTIP({ "id": "anonymox-infopanel" },
					VBOX(
						HBOX(
							LABEL({ "value": localize("infopanel.profile") }),
							LABEL({ "id": "anonymox-infopanel-profilelabel" })
						),
						HBOX(
							LABEL({ "value": "Service" }),
							LABEL({ "id": "anonymox-infopanel-servicelabel" })
						),
						BOX(
							BOX({ "style": "float: left; margin-right: 5px;" },
								IMAGE({
									"id"   : "anonymox-infopanel-flag",
									"style": "vertical-align: middle;"
								})
							),
							BOX(
								LABEL({
									"id"   : "anonymox-infopanel-iplabel",
									"value": ""
								})
							)
						)
					)
				),


				PANEL(
					{
						"id"            : "anonymox-panel",
						"onpopupshowing": "anonymox.events.onPanelOpened()",
						"onpopuphiding": "anonymox.events.onPanelClosed()"
					},
					MAIN(
						{
							"id": "anonymox-panel-main",
							"class": "settings-empty status-active"
						},
						SECTION({ "class": "main-settings" },
							DIV(
								{
									"class": "area area-settings",
									"onclick": "anonymox.events.onSiteSettingsClicked()"
								},
								DIV({ "class": "icon" }),
								DIV({ "class": "desc" },
									localize("panel.saveSettings")
								)
							)
						),
						SECTION({ "id": "anonymox-main-status", "class": "main-status" },
							DIV({
									"class": "area area-active-toggle",
									"onclick" : "anonymox.events.onActiveCheckboxClicked()"
								},
								DIV({ "class": "icon" }),
								DIV({ "class": "desc" },
									".."
								)
							),
							DIV({
								"class": "area area-menu-toggle",
								"onclick" : "anonymox.events.onShowCookieActionPicker()"
								},
								DIV({ "class": "icon" })
							),
							DIV({ "class": "area-menu" },
								DIV({ "class": "list" },
									DIV({
										"class": "entry",
										"onclick": "anonymox.events.onCookieActionSelected(window, 0)"
									},
										DIV({"class" : "name"}, localize("panel.cookies.noAction")),
										DIV({"class" : "icon selection"})
									),
									DIV({
										"class": "entry",
										"onclick": "anonymox.events.onCookieActionSelected(window, 1)"
									},
										DIV({"class" : "name"}, localize("panel.cookies.profileSite")),
										DIV({"class" : "icon selection"})
									),
									DIV({
										"class": "entry",
										"onclick": "anonymox.events.onCookieActionSelected(window, 2)"
									},
										DIV({"class" : "name"}, localize("panel.cookies.allSites")),
										DIV({"class" : "icon selection"})
									)
								)
							)
						),
						SECTION(
							{
								"id": "anonymox-panel-main-proxy",
								"class": "main-proxy"
							},
							DIV({
								"class": "area area-proxy-info country-us",
								"title": localize("panel.proxyinfo.title"),
								"onclick" : "anonymox.events.onShowIdentitiesList(window)"
							},
								DIV({ "class": "icon flag" }),
								DIV({ "class": "icon dropdown" }),
								DIV({ "class": "desc" },
									localize("panel.connectedvia")+": ",
									SPAN({ "class": "proxy-desc" },
										"-"
									)
								),
								DIV({ "class": "icon quality quality-1" })
							),
							DIV({
									"class": "area area-proxy-switch",
									"title": localize("panel.proxyswitch.title"),
									"onclick" : "anonymox.events.onChangeIdentityClicked(window)"
								},
								DIV({ "class": "icon" })
							)
						),
						SECTION(
							{
								"id": "anonymox-panel-main-connection-overview",
								"class": "main-connection-overview"
							},
							H2(localize("panel.connectionoverview")),
							DIV({ "class": "area area-zoom-toggle", "onclick" : "anonymox.events.onToggleShowDetails(window)" },
								DIV({ "class": "icon" })
							),
							DIV({ "class": "area-connection-graph local-proxy-target" },

								DIV({ "class": "connections" },
									DIV({ "class": "connection connection-local-proxy" }),
									DIV({ "class": "connection connection-proxy-target not-secured" }),
									DIV({ "class": "connection connection-local-target not-secured" })
								),
								DIV({ "class": "nodes" },
									DIV({ "class": "node node-local" }),
									DIV({ "class": "node node-proxy country-us" },
										DIV({ "class": "flag" })
									),
									DIV({ "class": "node node-target" })
								),
								DIV({ "class": "labels" },
									DIV({ "class": "label label-local" },
										DIV({ "class": "main" },
											SPAN({ "class": "name" },
												localize("panel.localdevice")
											)
										),
										DIV({ "class": "details" },
											SPAN({ "class": "ip" },
												"-"
											)
										)
									),
									DIV({ "class": "label label-proxy" },
										DIV({ "class": "main" },
											SPAN({ "class": "name" },
												"anonymoX ",
												SPAN({"class": "node-name"})
											)
										),
										DIV({ "class": "details" },
											SPAN({ "class": "ip" },
												"-"
											)
										)
									),
									DIV({ "class": "label label-target" },
										DIV({ "class": "main" },
											SPAN({ "class": "name" },
												"n/a"
											)
										),
										DIV({ "class": "details" },
											SPAN({ "class": "ip" },
												"-"
											)
										)
									)
								)
							)
						),
						SECTION({ "class": "main-footer" },
							DIV(
								{
									"id": "anonymox-main-premium-status",
									"class": "area area-premium-status premium-active",
									"onclick": "anonymox.events.onPremiumInfoClicked()"
								},
								DIV({ "class": "icon" }),
								DIV({ "class": "desc" },
									"Premium status"
								)
							),
							DIV(
								{
									"class": "area area-premium-trial",
									"onclick": "anonymox.events.onPremTrialButtonClicked()",
									"style": "display:none;"
								},
								DIV({ "class": "desc" },
									SPAN({ "class": "highlight"}, localize("panel.premtrial.highlight")),
									" " + localize("panel.premtrial.text")
								)
							),
							DIV(
								{
									"class": "area area-help",
									"onclick": "anonymox.events.onContactButtonClicked()"
								},
								DIV({ "class": "icon" })
							),
							DIV(
								{
									"class": "area area-settings",
									"onclick": "anonymox.events.onOpenPreferences(window)"
								},
								DIV({ "class": "icon" })
							)
						),
						DIV(
							{
								"class": "overlay-identities",
								"style": "display:none;"
							},
							DIV(
								{
									"class": "overlay-wrapper"
								},
								DIV({ "class": "header" },
										DIV({
											"class": "close-button",
											"onclick" : "anonymox.events.onHideIdentitiesList(window)"
										}),
										DIV(
											{ "class": "title countries" },
											localize("panel.country")
										),
										DIV(
											{ "class": "title identities" },
											localize("panelIdentitiesLabel"),
											SPAN({ "class": "count"}, "")
										)
								),
								DIV({ "class": "content" },
									DIV(
										{
											"class": "column countries",
											"onclick": "anonymox.events.onCountrySelected(event)"
										}
									),
									DIV({ "class": "column identities" })
								)
							),
							DIV({ "class": "overlay-bg" })
						)
					)
				)
			),
			KEYSET(
				KEY({
					"id"       : "anonymox-openPanelKey",
					"modifiers": "alt control",
					"key"      : "x",
					"oncommand": "anonymox.events.onOpenPanel(window, 'toolsMenu', event)",
				})
			)
		];

		nodes.forEach(function(node) {
			doc.documentElement.appendChild(node.build(doc));
		});

		let toolsMenuEntry =
			MENU(
				{
					"id"       : "anonymox-toolsMenuEntry",
					"label"    : localize("toolsMenuEntryName"),
					"accesskey": localize("toolsMenuEntryName.accessKey")
				},
				MENUPOPUP(
					MENUITEM({
						"label"    : localize("toolsMenuEntry.showPanel"),
						"key"      : "anonymox-openPanelKey",
						"accesskey": localize("toolsMenuEntry.showPanel.accessKey"),
						"oncommand": "anonymox.events.onOpenPanel(window, 'toolsMenu', event)"
					}),
					MENUITEM({
						"label"    : localize("prefs"),
						"oncommand": "anonymox.events.onOpenPreferences(window)"
					}),
					MENUITEM({
						"label"    : localize("toolsMenuEntry.reloadIP"),
						"accesskey": localize("toolsMenuEntry.reloadIP.accessKey"),
						"oncommand": "anonymox.events.onExternalInfoRefreshRequested(window)"
					})
				)
			);
		let toolsMenu = doc.getElementById("menu_ToolsPopup");
		toolsMenuEntry.build(toolsMenu);
	},

	clearAll: function(doc) {
		let info = new Info("domBuilder.clearAll");
		const ids = [
			"anonymox-adbox",
			"anonymox-infopanel",
			"anonymox-panel",
			"anonymox-openPanelKey",
			"anonymox-toolsMenuEntry"
		];

		ids.forEach(function(id) {
			const node = doc.getElementById(id);
			try {
				node.parentNode.removeChild(node);
			} catch (e) {
				info.debug("can't clear " + id);
			}
		})



		CustomizableUI.destroyWidget("anonymox-view-button");
		CustomizableUI.removeListener(domBuilder.customizableUIListener);
	},

	customizableUIListener: {
		onWidgetAdded : function(widgetId, area, pos) {













			events.onUiUpdateRequired();
		}
	}
};
