/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/
"use strict";





















var events = {




	onWindowAddressUpdate : function() {

		ui.safeWindowIterator(function(chromeWindow) {
			var uri = chromeWindow.getBrowser().currentURI.spec;

			if (attachedData.get("uri", chromeWindow) !== uri) {
				attachedData.set("uri", chromeWindow, uri);

				events.onCurrentTabAddressUpdate(chromeWindow, uri);
			}
		});
	},





	onCurrentTabAddressUpdate : function(chromeWindow, uri) {
		const currentProfile = profilemgr.getProfileForURI(uri);
		attachedData.set("profile", chromeWindow, currentProfile);

		ui.locationOrProfileChange(chromeWindow);
		ui.updatePanel();
		ui.profiles.onProfileIsBeingUsed(currentProfile);
		geoLocation.updateFakeApiResponse(currentProfile);
	},


	onProfileIsBeingUsed: function(profile) {
		ui.profiles.onProfileIsBeingUsed(profile);
	},


	onExternalInfoRefreshRequested: function(win) {
		if (!state.isReady) {
			return;
		}

		let info = new Info("events.onExternalInfoRefreshRequested");
		core.getExternalInfo(win.anonymoxdata.profile.gateway, true);
		info.sendMsg(utils.msgType.usage, "features", "refreshExternalInfo");
	},

	onNewWindowOpened: function(chromeWindow) {
		const tabBrowser = chromeWindow.gBrowser.selectedBrowser;
		const uri        = tabBrowser.currentURI.asciiSpec;
		const profile    = profilemgr.getProfileForURI(uri);
		attachedData.set("profile", chromeWindow, profile);

		ui.newWindowOpened(chromeWindow);
	},


	onCloseAdBoxClicked: function() {
		ui.hideAdBoxIfOpen();
	},


	onDisableAdBoxClicked: function() {
		ui.adBoxPanel.hidePopup();
		browserapi.openURI(config.disableAdsURI, 0);
	},


	onPremiumInfoClicked: function() {
		browserapi.openURI(config.premiumInfoURI + "premiumstatus", 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},


	onPremTrialButtonClicked: function() {
		browserapi.openURI(config.premiumTrialUri, 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},


	onClearCookiesClicked: function(win) {
		ui.clearCookiesAsker(win);
	},


	onCookieSettingSelected: function(win, index) {
		browserapi.acceptCookies(index, "acceptCookiesDeleteQuestion");

		if (index == 1 || index == 2) {
			ui.notification(0, "disableCookiesDeleteHint");
		}
	},

	onUserAgentSettingSelected: function(obj) {
		ui.setUserAgent(obj);
	},

	onHideReferrerClicked: function() {
		ui.toggleReferrer();
	},


	onPanelOpened: function() {
		ui.panelWindow.anonymox.panel.onPopupShowingHandler();
	},


	onPanelClosed: function() {
		ui.panelWindow.anonymox.panel.hideIdentitiesList();
	},



	onNetworkStatusClicked: function() {
		browserapi.openURI(state.displayStatusURI, 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},


	onSiteSettingsClicked: function() {
		ui.panelWindow.anonymox.panel.siteSettingsCmd();
	},


	onProfilesWarningClicked: function() {
		browserapi.openURI(config.profilesWarnUri, 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},


	onActiveCheckboxClicked: function() {
		ui.panelWindow.anonymox.panel.activeButtonClick();
	},


	onChangeIdentityClicked: function(win) {
		win.anonymoxdata.profile.rotateGateway();
	},


	onCountrySelected: function(event) {

		ui.panelWindow.anonymox.panel.countryListCmd(event.target.parentNode);
	},


	onIdentitiesHelpClicked: function() {
		browserapi.openURI(config.identListHelpUri, 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},

	onShowCookieActionPicker: function() {
		ui.panelWindow.anonymox.panel.toggleCookieActionPicker();
	},

	onToggleShowDetails: function(win) {
		win.anonymox.panel.toggleDetails();
	},


	onCookieActionSelected: function(win, action) {
		win.anonymoxdata.profile.setCookieAction(action);
		ui.panelWindow.anonymox.panel.toggleCookieActionPicker();
	},

	onContactButtonClicked: function() {
		browserapi.openURI(config.supportFeedbackPageUri, 0);
		ui.panelWindow.anonymox.panel.closePanel();
	},

	onOpenPreferences: function(win) {
		win.anonymox.panel.showPreferences(win);
	},

	onShowIdentitiesList: function(win) {
		win.anonymox.panel.showIdentitiesList();
	},

	onHideIdentitiesList: function(win) {
		win.anonymox.panel.hideIdentitiesList();
	},

	onOpenPanel: function(win, origin) {
		win.anonymox.panel.open(origin);
	},

	onClosePanel: function(win) {
		win.anonymox.panel.closePanel();
	},




	onCommonDialogOpened: function(win) {
		overrideProxyCredentialDialog(win);
	},





	onShareLocationDialogOpened: function(shareLocationDialog) {
		geoLocationUi.updateShareLocationDialog(shareLocationDialog);
	},







	onSwitchOff: function() {
		geoLocation.clearFakeApiResponse();
		ui.deactivate();
		config.prefListener.unregister();
	},

	onExternalInfoUpdated: function(gateway, ani) {
		ui.externalInfoUpdated(gateway, ani);
		ui.updateUIElements("extInfo");
	},

	onPremiumActivated: function() {
		ui.updateUIElements("icons");
		ui.hideAdBoxIfOpen();
		ui.rebuildPremiumInfoParts(undefined);
	},

	onPremiumDeactivated: function() {
		ui.updateUIElements("icons");
	},

	onRefreshInfo: function(result) {
		ui.rebuildPremiumInfoParts(result.showPremiumInfoParts);


		if(result.throttledReminderIntervalMin != ui.throttledReminderIntervalMin){
			ui.throttledReminderIntervalMin = result.throttledReminderIntervalMin;
			ui.updateThrottledReminderTimer();
		}
	},

	onAddOnUpdateRequested: function(version) {
		ui.updateRequest(version);
	},





	onProfileAdded: function(profile) {
		ui.profileChangedHook("add", profile);
	},


	onProfileRemoved: function(profile) {
		ui.profileChangedHook("remove", profile);
	},


	onProfileUpdated: function(profile) {
		ui.profileChangedHook("change", profile);
		geoLocation.updateFakeApiResponse(profile);
	},


	onNoMatchingGatewayFound: function(profile) {
		ui.profiles.onNoMatchingServiceFound(profile);
	},

	onGatewayChangedAutomatically: function(profile, oldGateway) {
		ui.profiles.onAutoGatewayChanged(profile, oldGateway);
	},




	onSystemStateUpdated: function(status) {
		ui.systemStateUpdateHook(status);
		geoLocationUi.updateShareLocationDialogIfOpen();
	},





	onUiConfigUpdated: function() {
		if (state.isReady) {
			core.getAllExternalInfoIfNeeded();
			ui.updateUIElements("extInfo");
			ui.updateUIElements("statusBarEntry");
		}
	},














	onNotifyUser: function(level, messageId, messageArgs, buttons, onlyCurrentTab) {
		ui.notification(
			level,
			messageId,
			messageArgs,
			buttons,
			onlyCurrentTab
		);
	},






	onNotifyUserModal: function(message, args) {
		ui.notifyModal(localize(message, args));
	},





	onRemoveNotificationForAllWindows: function(value) {
		ui.removeNotificationForAllWindows(value);
	},



	onUiUpdateRequired: function() {
		ui.updateUIElements();
		ui.updateStatusLabels();
		ui.safeWindowIterator(function(win) {
			ui.updateExternalInfo(win);
		});
	},
}
