/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/
"use strict";


const geoLocation = {


	updateFakeApiResponse: function(profile) {
		const info = new Info("geoLocation.updateFakeApiResponse");

		const chromeWindow = Services.wm.getMostRecentWindow("navigator:browser");
		if (!chromeWindow) {
			info.debug("Window is not ready yet. Not updating geolocation response.");
			return;
		}

		if (attachedData.get("profile", chromeWindow) !== profile) {



			return;
		}

		if (!profile.fakeGeolocation) {
			info.debug("Profile is configured not to fake geolocation.");
			geoLocation.clearFakeApiResponse();
			return;
		}

		const countryCode = profile.gateway.country;

		const locations = {
			"at": '{"lat":48.2,"lng":16.366667}',
			"au": '{"lat":-35.3075,"lng":149.124417}',
			"br": '{"lat":-15.793889,"lng":-47.882778}',
			"ca": '{"lat":45.416667,"lng":-75.683333}',
			"ch": '{"lat":46.95,"lng":7.45}',
			"de": '{"lat":52.516667,"lng":13.383333}',
			"es": '{"lat":40.383333,"lng":-3.716667}',
			"fr": '{"lat":48.8567,"lng":2.3508}',
			"hk": '{"lat":22.3,"lng":114.2}',
			"it": '{"lat":41.9,"lng":12.5}',
			"nl": '{"lat":52.366667,"lng":4.9}',
			"pl": '{"lat":52.233333,"lng":21.016667}',
			"se": '{"lat":59.329444,"lng":18.068611}',
			"uk": '{"lat":51.507222,"lng":-0.1275}',
			"us": '{"lat":38.904722,"lng":-77.016389}',
			"za": '{"lat":-26.1714402,"lng":28.0050052}'
		};
		const location = locations[countryCode];

		if (!location) {
			info.debug("Profile returned no location. Clearing override.");
			geoLocation.clearFakeApiResponse();
			return;
		}

		info.debug("Using location: " +location+ " (" +countryCode+ ")");

		const value = 'data:application/json,{"location":' +location+ ',"accuracy":10}';
		Services.prefs.setCharPref("geo.wifi.uri", value);
	},


	clearFakeApiResponse: function() {
		Services.prefs.clearUserPref("geo.wifi.uri");
	}
}
