/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";


var profilemgr = {

	nullGateway : undefined,

	list : new Array(),

	getCountryName : function(country) {
		return (country ? country : "all");
	},

	getACEHost : function(host) {
		var idnService = Components.classes["@mozilla.org/network/idn-service;1"]
			.getService(Components.interfaces.nsIIDNService);

		let ACEHost = host;
		if (!idnService.isACE(host)) {
			ACEHost = idnService.convertUTF8toACE(host);
		}
		return ACEHost;
	},

	getReadableName : function(site) {
		var idnService = Components.classes["@mozilla.org/network/idn-service;1"]
				.getService(Components.interfaces.nsIIDNService);

		let displayName;
		if (idnService.isACE(site)) {
			displayName = idnService.convertACEtoUTF8(site);
		} else {
			displayName = site;
		}

		return displayName.length < 26 ? displayName : displayName
				.substr(0, 24)
				+ "..";
	},

	uriFilterType : {
		standard : 0,
		custom : 1
	},

	defaultFilter : {},

	init : function() {
		var info = new Info("profilemgr.init");

		profilemgr.nullGateway = {
			id : localize("null_gatewayName"),
			nullGw : true
		};

		profilemgr.list = new Array();

		profilemgr.defaultFilter = {
			enabled : true,
			attributes : {}
		};

		var commChannel = new Profile;
		commChannel.name = "commChannel";
		commChannel.uriFilterType = profilemgr.uriFilterType.custom;
		commChannel.uriFilter = "^" + utils.escapeForRegex(config.masterURI) + "$";
		commChannel.isInternal = true;
		commChannel.gateway = profilemgr.nullGateway;
		commChannel.gatewayFilter.enabled = false;
		commChannel.registerProfile();

		var localChannel = new Profile;
		localChannel.name = "local";
		localChannel.uriFilterType = profilemgr.uriFilterType.custom;
		localChannel.uriFilter = /^(?:https?|ftp|wss?):\/\/(?:localhost|fritz\.box|easy\.box|[^\/]*\.spotilocal.com|127\.0\.0\.1|192\.168\.\d*\.\d*|10\.\d*\.\d*\.\d*|172\.(?:1[6-9]|2[0-9]|3[0-1])\.\d*\.\d*)(?::\d+)?\/.*/;
		localChannel.isInternal = true;
		localChannel.gateway = profilemgr.nullGateway;
		localChannel.gatewayFilter.enabled = false;
		localChannel.registerProfile();

		var specialChannel = new Profile;
		specialChannel.name = "special";
		specialChannel.uriFilterType = profilemgr.uriFilterType.custom;
		specialChannel.uriFilter = /^(?:about:|chrome:|jar:file:\/\/|file:\/\/)/;
		specialChannel.isInternal = true;
		specialChannel.gateway = profilemgr.nullGateway;
		specialChannel.gatewayFilter.enabled = false;
		specialChannel.registerProfile();

		var anynetChannel = new Profile;
		anynetChannel.name = "anonymox.net";
		anynetChannel.site = "anonymox.net";
		anynetChannel.uriFilterType = profilemgr.uriFilterType.custom;
		anynetChannel.uriFilter = /^(?:https?):\/\/(?:anonymox\.net)|(?:curopayments\.net)\/.*/;
		anynetChannel.isInternal = true;
		anynetChannel.gateway = profilemgr.nullGateway;
		anynetChannel.gatewayFilter.enabled = false;
		anynetChannel.registerProfile();











		profilemgr.load();

		for (var i = 0; i < profilemgr.list.length; i++) {
			info.verbose("profile " + profilemgr.list[i].name + " : "
					+ profilemgr.list[i].uriFilter);
		}

		profilemgr.selfCheckTimer = setTimeout(
				profilemgr.selfCheckAllActiveGateways,
				1000 * 60 * config.selfCheckIntervalMin,
				Ci.nsITimer.TYPE_REPEATING_SLACK);
	},

	selfCheckAllActiveGateways : function(){
		let info = new Info("profilemgr.selfCheckAllActiveGateways"),
			num = 0,
			scheduledGateways = [];

		info.debug("");

		try {
			let createSelfcheckClosure = function(gateway) {
				return function() {
					try {
						if (!state.isReady)
							return;
						if (Array_inArray(core.gatewayListCache, gateway)
								|| gateway == profilemgr.nullGateway)
							core.selfCheckGW(gateway);
						else
							info.verbose("Gateway " + gateway.id
									+ " does not exist anymore, skipping test");
					} catch (e) {
						info.error(e);
					}
				};
			};

			let scheduleCheck = function(gateway) {
				if (scheduledGateways.indexOf(gateway) != -1)
					return;
				scheduledGateways.push(gateway);

				let nextInterval = 1000 * num++;

				gateway.selfcheckTimer = setTimeout(
						createSelfcheckClosure(gateway), nextInterval);
				info.verbose("Selfcheck for " + gateway.id + " scheduled in "
						+ nextInterval + " ms");
			};

			scheduleCheck(profilemgr.nullGateway);

			profilemgr.list.forEach(function(profile) {
				if (profile.gateway) {
					scheduleCheck(profile.gateway);
				}
			});

		} catch (e) {
			info.error(e);
		}
	},


	save : function(singleProfileToUpdate) {
		const info = new Info("profilemgr.save");

		const saveProps = [
			"name", "isInternal", "isPreset", "site", "uriFilterType",
			"uriFilter", "gatewayFilter", "cookieAction",
			"fakeGeolocation"
		];

		var beautify = function(val){
			var asNum = parseInt(val);
			if (asNum == val)
				return asNum;
			else
				return val;
		};

		var createSaveableProfile = function(profile) {
			var profileToSave = JSON.parse(JSON.stringify(profile));


			for ( var prop in profileToSave) {
				var found = false;

				for (var j = 0; j < saveProps.length; j++) {
					var forbProp = saveProps[j];
					if (forbProp == prop)
						found = true;
				}

				if (!found) {
					profileToSave[prop] = undefined;
				}
			}

			profileToSave.gatewayFilter.country = beautify(profileToSave.gatewayFilter.country);
			profileToSave.gatewayFilter.wantId = undefined;


			if (profileToSave.uriFilterType == profilemgr.uriFilterType.standard)
				profileToSave.uriFilter = undefined;
			return profileToSave;
		};

		if (singleProfileToUpdate) {

			var profileList = JSON.parse(prefs.getCharPref("profiles.list"));

			for (var i = 0; i < profileList.length; i++) {
				if (profileList[i].name == singleProfileToUpdate.name)
					profileList[i] = createSaveableProfile(singleProfileToUpdate);
			}
			info.verbose("Saved single profile " + singleProfileToUpdate.name);
			prefs.setCharPref("profiles.list", JSON.stringify(profileList));
		} else {

			var profilesToSave = new Array();
			for (var i = 0; i < profilemgr.list.length; i++) {
				var profile = profilemgr.list[i];
				if (!profile.isInternal)
					profilesToSave.push(createSaveableProfile(profile));
			}
			info.verbose("saved " + profilesToSave.length + " profiles");
			prefs.setCharPref("profiles.list", JSON.stringify(profilesToSave));
		}
	},


	load : function() {
		var info = new Info("profilemgr.load");

		try {
			var savedProfiles = JSON.parse(prefs.getCharPref("profiles.list"));

			for (var i = 0; i < savedProfiles.length; i++) {
				var savedProfile = savedProfiles[i];
				var newProfile = new Profile;
				for ( var prop in savedProfile) {
					newProfile[prop] = savedProfile[prop];
				}




				if (newProfile.gatewayFilter) {
					if (newProfile.gatewayFilter.type == 0){
						info.debug(newProfile.name + " update, gatewayFilter.type == 0 -> gatewayFilter.enabled=false and type=1");
						newProfile.gatewayFilter.enabled = false;
						newProfile.gatewayFilter.type = 1;
					} else if (newProfile.gatewayFilter.enabled == undefined) {
						info.debug(newProfile.name + " update, gatewayFilter.enabled==undef && gatewayFilter.type != 0 -> gatewayFilter.enabled=true");
						newProfile.gatewayFilter.enabled = true;
					}

					if (newProfile.gatewayFilter.pick){
						info.debug(newProfile.name + " update, pick!=undef -> gatewayFilter.pick = undefined");
						newProfile.gatewayFilter.pick = undefined;
					}
				} else {
					info.debug(newProfile.name + " update, !gatewayFilter -> creating standard profile");
					newProfile.gatewayFilter = profilemgr.defaultFilter;
				}
				newProfile.gatewayRotateHistory = null;




				if (newProfile.gatewayFilter.attributes == undefined) {
					info.debug(newProfile.name + " update: !gatewayFilter.attributes -> creating");
					newProfile.gatewayFilter.attributes = {};
				} else if (newProfile.gatewayFilter.attributes
						&& newProfile.gatewayFilter.attributes.length) {
					var newAttrib = {};
					info.debug(newProfile.name + " update: gatewayFilter.attributes  Array -> Object");
					for (var j = 0; j < newProfile.gatewayFilter.attributes.length; j++) {
						var upAt = newProfile.gatewayFilter.attributes[j];
						newAttrib[upAt] = true;
					}
					newProfile.gatewayFilter.attributes = newAttrib;
				}





				if (newProfile.gatewayFilter.country == 0)
					delete newProfile.gatewayFilter.country;
				delete newProfile.gatewayFilter.type;




				newProfile.assignUriFilter();
				profilemgr.list.push(newProfile);
			}
		} catch (e) {
			info.error(e);
		}

		info.debug("finished loading");

		profilemgr.save();
	},

	getProfileForURI : function(uri) {
		var info = new Info("profilemgr.getProfileForURI");

		try {

			let aceUri = uri;
			let domainParts = utils.getDomainPartsFromURI(uri);

			let matches = uri.match(/^(?:https?|ftp):\/\/([^:\/]*)/);
			if (matches != null) {
				var domain = matches[1];
				if (domainParts) {
					aceUri = uri.replace(domain, domainParts[0]);
				}
			}

			for (var i = 0; i < profilemgr.list.length; i++) {
				var profile = profilemgr.list[i];
				if (aceUri.match(profile.uriFilter)) {
					return profile;
				}

			}

			info.error("No matching profile found", "For " + uri);
			return undefined;
		} catch (e) {
			info.error(e);
		}
	},

	serializeFilterAttributes : function(attribs) {
		let ret = "{";
		let first = true;
		config.attributes.forEach(function(name) {
			if (attribs[name]) {
				if (first)
					first = false;
				else
					ret += ", ";
				ret += name + ":" + attribs[name];
			}
		});
		return ret + "}";
	},

	serializeFilter : function(filter) {
		return "{ "
				+ (filter.enabled ? "enabled:" + filter.enabled + " " : "")
				+ (filter.country ? "country:" + filter.country + " " : "")
				+ (filter.attributes.length > 0 ? "attributes:"
						+ profilemgr
								.serializeFilterAttributes(filter.attributes)
						+ " " : "")
				+ (filter.id ? "id:" + filter.id + " " : "")
				+ (filter.wantId ? "wantId " + filter.wantId + " " : "") + "}";
	},





	filterYieldsResults : function(gatewayFilter, site) {
		var info = new Info("profilemgr.filterYieldsResults");
		info.verbose("For " + (site == "*" ? "default" : profilemgr.getReadableName(site)) + ": " + profilemgr.serializeFilter(gatewayFilter));

		var tmpProfile = new Profile;
		tmpProfile.name = site == "*" ? "default" : site;
		tmpProfile.site = site;
		tmpProfile.uriFilterType = profilemgr.uriFilterType.standard;
		tmpProfile.gatewayFilter = gatewayFilter;
		let filteredGWs = tmpProfile.getGatewaysForFilter(gatewayFilter);
		return filteredGWs && filteredGWs.length > 0;
	},

	setProfile : function(gatewayFilter, site) {
		var info = new Info("profilemgr.setProfile");


		site = profilemgr.getACEHost(site);

		info.verbose("For " + site + " filter "
				+ profilemgr.serializeFilter(gatewayFilter));

		var profileToUpdate;


		for ( var i = 0; i < profilemgr.list.length; i++) {
			var profile = profilemgr.list[i];
			if (profile.site == site) {
				profileToUpdate = profile;
				break;
			}
		}


		if (!profileToUpdate) {
			info.debug("Adding " + site + " filter " + profilemgr.serializeFilter(gatewayFilter));
			profileToUpdate = new Profile;
			profileToUpdate.name = site;
			profileToUpdate.site = site;
			profileToUpdate.uriFilterType = profilemgr.uriFilterType.standard;
			profileToUpdate.gatewayFilter = gatewayFilter;
			profileToUpdate.assignGateway(true);
			profileToUpdate.registerProfile();
			events.onNotifyUser(0, "newProfileCreated", profilemgr.getReadableName(site));
		}else{
			info.debug("Updating Profile " + profileToUpdate.name + "(site " + site + ") filter " + profilemgr.serializeFilter(gatewayFilter));
			profileToUpdate.gatewayFilter = gatewayFilter;
			profileToUpdate.assignGateway(true);
			profilemgr.save(profileToUpdate);
		}
	},

	removeProfile : function(site) {
		var info = new Info("profilemgr.removeProfile");
		info.debug("for " + site);

		var found = false;
		for ( var i = 0; i < profilemgr.list.length; i++) {
			var profile = profilemgr.list[i];
			if (profile.site == site) {
				profile.removeFromProfileList();
				found = true;
				break;
			}
		}
		if (!found)
			throw ("E002: No profile to remove found");
	},


	assignGWForAll : function(){
		for ( var i = 0; i < profilemgr.list.length; i++) {
			if (!profilemgr.list[i].isTemporary)
				profilemgr.list[i].assignGateway(true, true);
		}
	}
};
