/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";


Cu.import("resource://gre/modules/Services.jsm");



var ui = {
	adBoxTimer : undefined,
	notifications : {},
	panelWindow : undefined,
	throttledReminderIntervalMin : undefined,
	throttledReminderTimer : undefined,
	viewLocation : undefined,




	newWindowOpened : function(win) {
		var info = new Info("newWindowOpened");
		info.verbose("");
		try{



			const currentURI = win.gBrowser.selectedBrowser.currentURI;
			attachedData.set("uri", win, currentURI.asciiSpec);


			loadStyleSheets([
				"chrome://anonymox/skin/toolbar-button.css",
				"chrome://anonymox/skin/style.css",
				"chrome://anonymox/skin/main.css",
				"chrome://anonymox/skin/flags.css"
			]);

			if (state.isReady)
				ui.activate(win);

			ui.initNotifications(win);
			ui.updateStatusLabels(win);
			ui.updateUIElements(null, win);
		}catch(e){
			info.error(e);
		}

		function loadStyleSheets(paths) {
			paths.forEach(function(path) {
				const styleSheetService =
					Cc["@mozilla.org/content/style-sheet-service;1"]
						.getService(Ci.nsIStyleSheetService);

				styleSheetService.loadAndRegisterSheet(
					Services.io.newURI(path, null, null),
					styleSheetService.AUTHOR_SHEET
				);
			})
		}
	},


	safeWindowIterator : function(func, forceAll){
		var browserEnumerator = Cc["@mozilla.org/appshell/window-mediator;1"].
			getService(Ci.nsIWindowMediator).getEnumerator("navigator:browser");
		while (browserEnumerator.hasMoreElements()) {
			var win = browserEnumerator.getNext();
			if (win.anonymoxdata || forceAll)
				func(win);
		}
	},


	activate : function(win) {
		var info = new Info("activate");
		info.verbose("");
		if (!win) {
			ui.safeWindowIterator(function(iwin){
				ui.activate(iwin);
			});
		} else {
			let browser = win.gBrowser.selectedBrowser;
			let uri     = browser.currentURI.asciiSpec;
			let profile = profilemgr.getProfileForURI(uri);

			attachedData.set("profile", win, profile);
			ui.profiles.onProfileIsBeingUsed(win.anonymoxdata.profile);

			ui.locationOrProfileChange(win);
			win.gBrowser.addProgressListener(browserapi.progressListener);
		}
	},

	deactivate : function(win) {
		var info = new Info("ui.deactivate");
		if (!win) {
			ui.safeWindowIterator(function(iwin) {
				ui.deactivate(iwin);
			});
		} else {
			ui.updateStatusLabels(win);
			win.gBrowser.removeProgressListener(browserapi.progressListener);
			win.anonymoxdata = undefined;

			unloadStyleSheets([
				"chrome://anonymox/skin/toolbar-button.css",
				"chrome://anonymox/skin/style.css",
				"chrome://anonymox/skin/main.css",
				"chrome://anonymox/skin/flags.css"
			]);


			for ( var i = 0; i < win.gBrowser.browsers.length; i++) {
				var browser = win.gBrowser.browsers[i];
				browser.anonymoxdata = undefined;
			}


			if (win.anonymox.panel) {
				win.anonymox.panel.closePanel();

				if (win.anonymox.panel.prefDialog) {
					win.anonymox.panel.prefDialog.close();
				}
			}
		}

		function unloadStyleSheets(paths) {
			paths.forEach(function(path) {
				const styleSheetService =
					Cc["@mozilla.org/content/style-sheet-service;1"]
						.getService(Ci.nsIStyleSheetService);

				styleSheetService.unregisterSheet(
					Services.io.newURI(path, null, null),
					styleSheetService.AUTHOR_SHEET
				);
			})
		}
	},



	externalInfoUpdated : function(gateway, ani){
		var info = new Info("ui.externalInfoUpdated");
		try{
			var update = function(){
				info.verbose(gateway.id + " " + gateway.externalInfo.status + " " + (gateway.externalInfo.status == 2 ? gateway.externalInfo.ip + "(" + gateway.externalInfo.country + ")": ""));

				ui.safeWindowIterator(function(iwin){
					if (iwin.anonymoxdata.profile && iwin.anonymoxdata.profile.gateway == gateway)
						ui.updateExternalInfo(iwin);
				});
			};

			if (ani){
				gateway.aniTime = setTimeout(function(){
					update();
				}, 300);
			}else
				update();
		} catch (e) {
			info.error(e);
		}
	},

	updateExternalInfo : function(win){
		var info = new Info("ui.updateExternalInfo");

		if (!win.anonymoxdata) return;
		let gateway = win.anonymoxdata.profile.gateway;
		info.verbose("gateway " + gateway.id + " " + (gateway.externalInfo?gateway.externalInfo.ip:"---"));

		var bar_iplabel = win.document.getElementById('anonymox-externalIp');
		var panel_iplabel = win.document.getElementById('anonymox-infopanel-iplabel');

		if (!gateway.externalInfo) {
			utils.setElementValue(panel_iplabel, "...");
			utils.setElementValue(bar_iplabel, "...");
		} else {
			let newVal = "";
			switch (gateway.externalInfo.status) {
				case 1:
					if (bar_iplabel && bar_iplabel.value == "") {
						newVal = localize("IPLoading");
						utils.setElementValue(panel_iplabel, newVal);
						utils.setElementValue(bar_iplabel, newVal);
					}
					if (panel_iplabel)
						panel_iplabel.style.color = "orange";
					if (bar_iplabel)
						bar_iplabel.style.color = "orange";
					break;
				case 2:
					newVal = localize("IP") + " " + gateway.externalInfo.ip;
					utils.setElementValue(panel_iplabel, newVal);
					utils.setElementValue(bar_iplabel, newVal);

					if (panel_iplabel)
						panel_iplabel.style.color = "green";
					if (bar_iplabel)
						bar_iplabel.style.color = "green";
					break;
				case -1:
					newVal = localize("IPLoadFailed");
					utils.setElementValue(panel_iplabel, newVal);
					utils.setElementValue(bar_iplabel, newVal);
					if (panel_iplabel)
						panel_iplabel.style.color = "red";
					if (bar_iplabel)
						bar_iplabel.style.color = "red";
					break;
			}
		}


		var bar_flag = win.document.getElementById('anonymox-externalFlag');
		var panel_flag = win.document.getElementById('anonymox-infopanel-flag');

		if (!gateway.externalInfo) {
			utils.setElementAttribute(panel_flag, "src", "");
			utils.setElementAttribute(bar_flag, "src", "");
		} else {
			switch (gateway.externalInfo.status) {
				case 1:
					utils.setElementAttribute(panel_flag, "src", "");
					utils.setElementAttribute(panel_flag, "src", "");
					break;
				case 2:
					utils.setElementAttribute(
						panel_flag, "src",
						"chrome://anonymox/skin/flags/" + gateway.externalInfo.country + ".png"
					);
					utils.setElementAttribute(
						bar_flag, "src",
						"chrome://anonymox/skin/flags/" + gateway.externalInfo.country + ".png"
					);
					break;
				case -1:
					utils.setElementAttribute(panel_flag, "src", "");
					utils.setElementAttribute(bar_flag, "src", "");
					break;
			}
		}
	},



	locationOrProfileChange : function(win) {
		var info = new Info("ui.locationOrProfileChange");

		try {
			const profile = win.anonymoxdata.profile;
			if (!profile) {
				info.error("Profile not set.");
				return;
			}


			ui.updateStatusLabels(win);
			ui.updateExternalInfo(win);


			if (state.adBoxInfo)
				ui.adBoxURIChange();

			ui.removeNonPersistantNotificationsForAllWindows();
		} catch(e) {
			info.error(e);
		}
	},



	profileChangedHook : function(topic, profile) {
		var info = new Info("ui.profileChangedHook");
		try {
			info.verbose(topic + " " + profile.name + " (gwid " + profile.gateway.id + ")");
			ui.safeWindowIterator(function(wind) {
				if (topic == "change") {
					if (wind.anonymoxdata.profile == profile) {
						ui.updateStatusLabels(wind);
						wind.anonymox.panel.profileChangedHook();
					}
				} else if (topic == "add" || topic == "remove") {
					let browser = wind.gBrowser.selectedBrowser;

					if (topic == "add") {
						wind.anonymoxdata.profile = profile;
					}
					else if (topic == "remove") {
						let uri            = browser.currentURI.asciiSpec;
						let currentProfile = profilemgr.getProfileForURI(uri);

						wind.anonymoxdata.profile = currentProfile;
					}

					ui.updateStatusLabels(wind);
					ui.updateExternalInfo(wind);

					ui.removeNonPersistantNotificationsForAllWindows();
				}

			});
		} catch(e) {
			info.error(e);
		}
	},



	adBoxInfoPageChangeCount : 0,

	adBoxURIChange : function() {
		if (!ui.dontAutoCloseAdBox)
			ui.adBoxInfoPageChangeCount++;

		if (ui.adBoxInfoPageChangeCount >= state.adBoxInfo.pageChangesPerAd) {
			ui.adBoxInfoPageChangeCount = 0;
			ui.showAdBox();
			ui.dontAutoCloseAdBox = true;
			ui.dontAutoCloseAdBoxTO = setTimeout(function(){ui.dontAutoCloseAdBox = false;}, config.adBoxNoAutoCloseIntervalSec * 1000);
		} else if (!ui.dontAutoCloseAdBox) {
			ui.hideAdBoxIfOpen();
		}
	},

	showAdBox : function() {
		let win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser"),
			info = new Info("ui.showAdBox"),
			width = win.document.width,
			height = win.document.height;











		if (!win.document.getElementById("anonymox-adbox-frame").webNavigation) {
			info.debug("Ad box not shown (workaraound for #829)");
			return;
		}

		if (win.anonymoxdata.profile.gateway != profilemgr.nullGateway) {
			ui.hideAdBoxIfOpen();
			ui.adBoxPanel_frame = win.document.getElementById("anonymox-adbox-frame");
			ui.adBoxPanel = win.document.getElementById("anonymox-adbox");

			if (ui.adBoxPanel_frame.getAttribute("src") != state.adBoxInfo.frameURL) {
				ui.adBoxPanel_frame.setAttribute("src", state.adBoxInfo.frameURL);
			} else {
				ui.adBoxPanel_frame.webNavigation.loadURI(state.adBoxInfo.frameURL, Ci.nsIWebNavigation.LOAD_FLAGS_NONE, null, null, null);
			}

			ui.adBoxPanel_frame.addEventListener("DOMContentLoaded", function() {
				if(ui.adBoxPanel.state=="closed") {
					ui.adBoxPanel.setAttribute("hidden", false);

					let framewidth = ui.adBoxPanel_frame.contentDocument.body.getAttribute("frame_width");
					let frameheight = ui.adBoxPanel_frame.contentDocument.body.getAttribute("frame_height");


					let contentWindow = win.document.getElementById("main-window");
					ui.adBoxPanel.openPopup(contentWindow, 'topleft', contentWindow.width / 2 - framewidth / 2, contentWindow.height - frameheight, false, false);


					if (width < framewidth * 2 || height < frameheight * 3) {
						info.debug("window is to small to display pop-up - closing.");
						ui.hideAdBoxIfOpen();
					}

					if (!framewidth || !frameheight) {
						ui.hideAdBoxIfOpen();
					} else {
						ui.adBoxPanel_frame.setAttribute("width", framewidth ? framewidth : 500);
						ui.adBoxPanel_frame.setAttribute("height", frameheight ? frameheight : 200);
					}
				}
			});

		}
	},

	hideAdBoxIfOpen : function() {
		if (ui.adBoxPanel)
			ui.adBoxPanel.setAttribute("hidden", true);
	},




	rebuildPremiumInfoParts : function(showPremiumInfoParts) {
		var info = new Info("rebuildPremiumInfoParts");
		info.verbose(showPremiumInfoParts);

		let parts = showPremiumInfoParts ? showPremiumInfoParts.split(",") : [];

		if (Array_inArray(parts, "morecountries")) {
			ui.showMoreCountries = true;
		} else {
			ui.showMoreCountries = false;
		}

	},

	showLimitNotification : function() {
		if(!state.isReady) return;

		var win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
		if(ui.throttledReminderIntervalMin && win.anonymoxdata.profile.gateway != profilemgr.nullGateway){
			ui.notification(1, "speedLimit", undefined,
				{
					label: localize("speedLimitUpdateButton"),
					accessKey: "u",
					callback: function() {
						browserapi.openURI(config.premiumInfoURI + "throttledReminder", 0);
					}
				}
			);
		}
		ui.updateThrottledReminderTimer();
	},

	updateThrottledReminderTimer : function() {
		var info = new Info("updateThrottledReminderTimer");
		if (!ui.throttledReminderIntervalMin) {
			info.debug("cancelling timer");
			if (ui.throttledReminderTimer)
				ui.throttledReminderTimer.cancel();
		} else {
			if (!ui.throttledReminderTimer) {
				info.debug("setting up timer to " + ui.throttledReminderIntervalMin + " min");
				ui.throttledReminderTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
				ui.throttledReminderTimer.initWithCallback({
					notify : function() {
						ui.showLimitNotification();
					}
				}, 1000 * 60 * ui.throttledReminderIntervalMin, Ci.nsITimer.TYPE_REPEATING_SLACK);
			} else if (ui.throttledReminderTimer.delay != 1000 * 60 * ui.throttledReminderIntervalMin) {
				ui.throttledReminderTimer.delay = 1000 * 60 * ui.throttledReminderIntervalMin;
				info.debug("timer interval updated to " + ui.throttledReminderIntervalMin + " min");
			}
		}
	},



	systemStateUpdateHook : function () {
		var info = new Info("ui.systemStateUpdated");

		try{
			info.debug(state.systemState.name);

			switch (state.systemState) {
				case state.systemStates.disabled:
					ui.updateUIElements("icons");
					ui.deactivate();
					break;
				case state.systemStates.ready:
					ui.updateUIElements("icons");
					ui.activate();

					break;
			}
			ui.updateStatusLabels();

		} catch (e){
			info.error(e);
		}
	},

	updateStatusLabels : function(win) {
		var info = new Info("ui.updateStatusLabels");

		try {
			if (!win) {
				ui.safeWindowIterator(function(iwin) {
					ui.updateStatusLabels(iwin);
				});
			} else {
				var statusLabel = win.document.getElementById('anonymox-statusLabel');
				var panel_profileLabel = win.document.getElementById('anonymox-infopanel-profilelabel');
				var panel_serviceLabel = win.document.getElementById('anonymox-infopanel-servicelabel');

				var statusColor;
				switch (state.systemState) {
					case state.systemStates.loading:
						statusColor = "orange";
						break;
					case state.systemStates.disabled:
						statusColor = "red";
						break;
				}

				if (state.systemState == state.systemStates.ready) {
					info.verbose("setting profile-based status");

					var profile = win.anonymoxdata.profile;

					let viewbutton = win.anonymox.jQuery(win.document.getElementById("anonymox-view-button"));

					if (core.isPremiumAccount && state.isReady) {
						viewbutton.addClass("premium");
					} else {
						viewbutton.removeClass("premium");
					}

					if (profile.isInternal) {
						statusColor = "blue";
						viewbutton.addClass("off");
					} else if (profile.gateway != profilemgr.nullGateway) {
						statusColor = "green";
						viewbutton.removeClass("off");
					} else {
						statusColor = "red";
						viewbutton.addClass("off");
					}

					if (profile.name == "default") {
						utils.setElementValue(statusLabel, profile.gateway.id);
					} else {
						utils.setElementValue(statusLabel, profile.getDisplayName() + " | " + profile.gateway.id);
					}

					utils.setElementValue(panel_profileLabel, profile.getDisplayName());
					utils.setElementValue(panel_serviceLabel, profile.gateway.id ? profile.gateway.id : "null");
					ui.updateUIElements("statusBarEntry", win);
				} else {
					info.verbose("setting status from global");

					utils.setElementValue(statusLabel, state.systemState.tString);
					utils.setElementValue(panel_profileLabel, state.systemState.tString);
					utils.setElementValue(panel_serviceLabel, "-");
				}


				if (statusLabel) {
					statusLabel.style.color = statusColor;
				}

				if (panel_profileLabel) {
					panel_profileLabel.style.color = statusColor;
					panel_serviceLabel.style.color = statusColor;
				}
			}
		} catch (e) {
			info.error(e);
		}
	},


	updateUIElements : function(key, win) {
		var info = new Info("ui.updateUIElements");

		if (!win) {
			ui.safeWindowIterator(function(iwin) {
				ui.updateUIElements(key, iwin);
			});
		} else {
			var doc = win.document;

			if (key == "icons" || !key) {
				ui.updatePremiumStatusIcon(win, core.isPremiumAccount);

				let viewbutton = win.anonymox.jQuery(win.document.getElementById("anonymox-view-button"));

				if (core.isPremiumAccount && state.isReady) {
					viewbutton.removeClass("disabled").addClass("premium");
				} else if (state.isReady) {
					viewbutton.removeClass("disabled").removeClass("premium");
				} else {
					viewbutton.addClass("disabled");
				}
			}

			if (key == "statusBarEntry") {
				var statusBarEntry = doc.getElementById('anonymox-statusBarEntry');
				if(statusBarEntry) statusBarEntry.hidden = !prefs.getBoolPref("displayInStatusBar");
			}

			if (key == "extInfo" || !key) {
				var statuslabel = win.document.getElementById('anonymox-statusLabel');
				var bar_iplabel = doc.getElementById('anonymox-externalIp');
				var panel_iplabel = doc.getElementById('anonymox-infopanel-iplabel');

				setDisplay(statuslabel, prefs.getBoolPref("displayGateway"));
				setDisplay(bar_iplabel, prefs.getBoolPref("displayIP"));
				setDisplay(panel_iplabel, true);

				var bar_flag = doc.getElementById('anonymox-externalFlag');
				var panel_flag = doc.getElementById('anonymox-infopanel-flag');

				setDisplay(bar_flag, prefs.getBoolPref("displayFlag"));
				setDisplay(panel_flag, true);
			}
		}


		function setDisplay(element, shouldDisplay) {
			if (!element) {



				return;
			}

			if (shouldDisplay) {
				element.style.display = "";
			}
			else {
				element.style.display = "none";
			}
		}
	},

	updatePremiumStatusIcon: function(win, isActive) {
		let premiumStatus = win.anonymox.jQuery("#anonymox-main-premium-status");
		let premiumStatusLabel = premiumStatus.children(".desc");
		if (isActive) {
			premiumStatus.removeClass("premium-inactive").addClass("premium-active");
			premiumStatusLabel.text(localize("panel.premium.active"));
		} else {
			premiumStatus.removeClass("premium-active").addClass("premium-inactive");
			premiumStatusLabel.text(localize("panel.premium.inactive"));
		}
	},



	initNotifications : function(win) {
		var info = new Info("initNotifications");

		win.gBrowser.getNotificationBox().notificationsHidden = false;


		for (var n in ui.notifications) {
			try {
				let currentNotification = ui.notifications[n];
				if (currentNotification.global) {
					ui.addNotificationForWindow(win, currentNotification);
				}
			} catch(e) {
				info.debug(e);
			}
		}
	},


	notification : function(level, messageid, messageargs, buttons, onlyCurrentTab) {
		let info = new Info("notification");

		let notificationPrefix = "showNotifications";


		if (level == 0) {
			if (prefs.getPrefType(notificationPrefix + "." + messageid) == prefs.PREF_INVALID) {
				info.error("pref " + notificationPrefix + "." + messageid + " not found");
			} else if (!prefs.getBoolPref(notificationPrefix + "." + messageid)) {
				info.debug("messageid: " + messageid + " not displayed per settings");
				return;
			}
		}

		let notificationID = messageid;

		if (!ui.notifications[notificationID]) {
			let debugMessage = "adding notification: " + notificationID;

			let buttonsArray = [];

			if (Array_isArray(buttons)) {
				buttonsArray = buttons;
			} else if (buttons) {
				buttonsArray = [buttons];
			}


			if (level == 0) {
				debugMessage += " with hide option";
				buttonsArray.push(
					{
						label: localize("neverShowAgain"),
						accessKey: "",
						callback: function() {
							prefs.setBoolPref(notificationPrefix + "." + messageid, false);
						}
					}
				);
			}

			var currentNotification = {
				label : localize(messageid, messageargs),
				value : notificationID,
				icon : 'chrome://anonymox/skin/icon16.png',
				priority : (onlyCurrentTab) ? "PRIORITY_INFO_HIGH" : "PRIORITY_INFO_MEDIUM",
				buttons : buttonsArray,
				global : (onlyCurrentTab) ? false : true
			};

			ui.notifications[notificationID] = currentNotification;

			if (onlyCurrentTab) {
				debugMessage += " to current tab";
				info.debug(debugMessage);
				ui.addNotificationForWindow(onlyCurrentTab, currentNotification);
			} else {
				debugMessage += " to all tabs";
				info.debug(debugMessage);

				ui.safeWindowIterator(function (window) {
					ui.addNotificationForWindow(window, currentNotification);
				});
			}
		} else {
			info.debug("notification already pending: " + notificationID);
		}
	},

	addNotificationForWindow : function(window, notification) {
		let info = new Info("addNotificationForWindow");

		let notifyBox = window.gBrowser.getNotificationBox();
		if (notifyBox) {
			notifyBox.appendNotification(
				notification.label,
				notification.value,
				notification.icon,
				notifyBox[notification.priority],
				notification.buttons,
				ui.notificationCallbackMaker(notification.value)
			);
		} else {
			info.debug("notifyBox not found");
		}
	},

	removeNotificationForAllWindows : function (value) {
		if (ui.notifications[value]) {
			delete ui.notifications[value];
			ui.safeWindowIterator(function (window) {
				ui.removeNotificationForWindow(window, value);
			});
		}
	},

	notificationCallbackMaker : function (value) {
		return function (type) {
			if (ui.notifications[value]) {
				switch (type) {
					case "removed":
						delete ui.notifications[value];
						ui.safeWindowIterator(function (window) {
							ui.removeNotificationForWindow(window, value);
						});
				}
			}
		};
	},

	removeNotificationForWindow : function(window, value) {
		let info = new Info("removeNotificationForWindow");

		let notifyBox = window.gBrowser.getNotificationBox();

		if (notifyBox) {
			let notification = notifyBox.getNotificationWithValue(value);
			if (notification) {

				try {
					notifyBox.removeNotification(notification);
				} catch (e) {}
			}
		} else {
			info.debug("notifyBox not found");
		}
	},

	removeNonPersistantNotificationsForAllWindows : function() {
		for (var n in ui.notifications) {
			let currentNotification = ui.notifications[n];
			if (!currentNotification.global) {
				ui.safeWindowIterator(function (window) {
					ui.removeNotificationForWindow(window, currentNotification.value);
				});
			}
		}
	},

	notifyModal : function(text) {
		var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
		promptService.alert(null, "anonymoX", text);
	},



	toggleReferrer : function() {
		let info = new Info("toggleReferrer");
		if (browserapi.isReferrerEnabled()) {
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", localize("confirmDeactivateReferrer")))
				browserapi.setReferrer(false);
		}
		else {
			browserapi.setReferrer(true);
		}

		info.sendMsg(utils.msgType.usage, "features", "toggleReferrer");
	},

	setUserAgent : function(obj) {
		if (obj.value != "[[DEFAULT]]") {
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", localize("confirmChangeUA"))) {
				browserapi.setUserAgent(obj.value);
			}
		}
		else {
			browserapi.setUserAgent(undefined);
		}
	},



	clearCookiesAsker : function(win) {
		var domain = utils.getDomainPartsFromURI(win.anonymoxdata.uri);
		if (domain == null) return;
		domain = domain[0];

		ui.notification(1, "confirmClearCookies", undefined,
			[
				{
					label: localize("confirmClearCookies_onlyThisPage", domain),
					accessKey: "O",
					callback: function() {
						browserapi.clearCookies(domain, "menu");
					}
				},
				{
					label: localize("confirmClearCookies_allPages"),
					accessKey: "A",
					callback: function() {
						browserapi.clearCookies(undefined, "menu");
					}
				}
			]
		);
	},




	updateRequest : function(version){
		var info = new Info("updateRequest");

		setTimeout(function() {
			ui.notification(1, "updateInstallQ", undefined,
				{
					label: localize("ok"),
					accessKey: "Y",
					callback: function() {
						ui.getInstall(version, function(install) {
							install.addListener(ui.installListener);
							install.install();
						})
					}
				}
			);
		}, 2000);
	},

	getInstall: function(version, callback) {
		Cu.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.getInstallForURL(
			"https://update.anonymox.net/anonymox-update-" + version + ".xpi",
			callback,
			"application/x-xpinstall"
		);

	},

	installListener: {
		onInstallEnded: function(install) {
			install.removeListener(ui.installListener);
		},
		onInstallFailed: function(install) {
			ui.notifyModal("Update failed. Please update manually from www.anonymox.net");
			install.removeListener(ui.installListener);
		},
		onDownloadFailed: function(install) {
			ui.notifyModal("Update failed. Please update manually from www.anonymox.net");
			install.removeListener(ui.installListener);
		},
		onInstallCancelled: function(install) {
			install.removeListener(ui.installListener);
		}
	},

	initUsageCollector : function() {
		var info = new Info("initUsageCollector");
		try {
			if (prefs.prefHasUserValue("usagecollection")) {
				var prefVal = prefs.getIntPref("usagecollection");
				if (prefVal != 0 && prefVal != 1) {
					if (new Date().getTime() / 1000 > prefVal) {

						if (!ui.initUsageCollectorTO){
							ui.initUsageCollectorTO = setTimeout(function() {
								ui.notification(1, "usageCollectionQuestion", undefined,
									[
										{
											label: localize("yes"),
											accessKey: "Y",
											callback: function() {
												prefs.setIntPref("usagecollection", 1);
											}
										},
										{
											label: localize("noDontAskAnymore"),
											accessKey: "N",
											callback: function() {
												prefs.setIntPref("usagecollection", 0);
											}
										}
									]
								);
								ui.initUsageCollectorTO = undefined;
							},
							1000 * 5
							);
						}
					}
				}
			} else {
				prefs.setIntPref("usagecollection", new Date().getTime() / 1000 + 60 * 60 * config.usageCollectionIntervalHrs);
			}
		} catch(e) {
			info.error(e);
		}
	},

	updatePanel: function() {
		if (ui.panelWindow) {
			ui.panelWindow.anonymox.panel.profileChangedHook();
		}
	},


	profiles: {
		showNoMatchingServiceFound: function(profile) {
			if (!profile.hideNoMatchingServiceFoundTakeDefault){
				events.onNotifyUser(1, "noMatchingServiceFoundTakeDefault", profile.name, [
					{
						label: "Ignore for session",
						accessKey: "i",
						callback: function() {
							profile.hideNoMatchingServiceFoundTakeDefault = true;
						}
					},
					{
						label: "Delete profile",
						accessKey: "d",
						callback: function() {
							profile.removeFromProfileList();
						}
					}
				]);
			}
		},

		showAutoGatewayChanged: function(profile){
			ui.notification(0, "autoGatewayChanged", profile.uiData.autoGatewayChangedArgs);
		},

		showPostponedMessage: function(profile) {
			let info = new Info("ui.profile.showPostponedMessage");

			if (profile.uiData.noMatchingServiceFoundMsgPostponed){
				profile.uiData.noMatchingServiceFoundMsgPostponed = false;
				if (profile.uiData.autoGatewayChangedMsgPostponed){
					profile.uiData.autoGatewayChangedMsgPostponed = false;
					info.debug("Dropping postponed autoGatewayChangedMsg");
				}
				ui.profiles.showNoMatchingServiceFound(profile);
			}

			if (profile.uiData.autoGatewayChangedMsgPostponed){
				info.debug("Displaying postponed autoGatewayChangedMsg");
				profile.uiData.autoGatewayChangedMsgPostponed = false;
				ui.profiles.showAutoGatewayChanged(profile);
			}
		},

		profileIsBusy: function(profile) {
			return new Date().getTime() - profile.uiData.lastUsedTime < config.siteIsConsideredBusyIntervalMS;
		},




		onNoMatchingServiceFound: function(profile) {
			let info = new Info("ui.profile.onNoMatchingServiceFound");

			if (ui.profiles.profileIsBusy(profile)) {
				ui.profiles.showNoMatchingServiceFound(profile);
			} else {
				info.debug("postponing noMatchingServiceFound");
				profile.uiData.noMatchingServiceFoundMsgPostponed = true;
			}
		},

		onAutoGatewayChanged: function(profile, oldGateway) {
			let info = new Info("ui.profile.onAutoGatewayChanged");

			profile.uiData.autoGatewayChangedArgs = [profile.name, oldGateway.id];
			if (ui.profiles.profileIsBusy(profile))
				ui.profiles.showAutoGatewayChanged(profile);
			else{
				info.debug("postponing autoGatewayChanged");
				profile.uiData.autoGatewayChangedMsgPostponed = true;
			}
		},

		onProfileIsBeingUsed: function(profile) {
			profile.uiData.lastUsedTime = new Date().getTime();
			ui.profiles.showPostponedMessage(profile);
		}
	}
};
