/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/


"use strict";

const HTML_NS = "http://www.w3.org/1999/xhtml";
const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";


var Xul = {};

function XulTag(tagName, html) {
	this.isHtml = html;
	this.tagName = tagName;
	this.children = [];
}


XulTag.prototype =

{
	merge: function(args) {

		let attrs = args.length ? args[0] : null;
		let hasAttrs = typeof(attrs) == "object" && !isTag(attrs);



		if (hasAttrs) {
			this.attrs = attrs;
		}


		if (args.length) {
			this.children = Array.prototype.slice.call(args);

			if (hasAttrs) {
				this.children.shift();
			}
		}

		return this;
	},

	build: function(parentNode, options = {}) {
		let doc = parentNode.ownerDocument || parentNode;


		let node = doc.createElementNS(this.isHtml ? HTML_NS : XUL_NS, this.tagName);
		for (let key in this.attrs) {
			node.setAttribute(key, this.attrs[key]);
		}


		for (let i=0; i<this.children.length; i++) {
			let child = this.children[i];

			if (typeof child === "string") {
				const textNode = doc.createTextNode(child);
				node.appendChild(textNode);
			}
			else {
				child.build(node);
			}
		}

		if (doc === parentNode) {
			return node;
		}



		if (options.insertBefore) {
			parentNode.insertBefore(node, options.insertBefore);
		}
		else {
			parentNode.appendChild(node);
		}

		return node;
	}
}

function isTag(obj) {
	return (obj instanceof XulTag);
}


function defineTags() {
	for (let i=0; i<arguments.length; i++) {
		let tagName = arguments[i];
		let html = (tagName.indexOf("html:") == 0);
		if (html) {
			tagName = tagName.substring(5);
		}
		let fn = createTagHandler(tagName, html);
		let fnName = tagName.toUpperCase();

		Xul[fnName] = fn;
	}

	function createTagHandler(tagName, html=false) {
		return function() {
			let newTag = new XulTag(tagName, html);
			return newTag.merge(arguments);
		};
	}
}


defineTags(
	"box", "vbox", "hbox", "splitter", "toolbar", "radio", "image",
	"menupopup", "textbox", "tabbox", "tabs", "tabpanels", "toolbarbutton",
	"arrowscrollbox", "tabscrollbox", "iframe", "description", "panel",
	"label", "progressmeter", "resizer", "stack", "spacer", "toolbaritem",
	"keyset", "key", "button", "popupset", "tooltip", "menulist", "checkbox",
	"richlistbox", "menuitem", "listheader", "listhead", "listcol", "listcols",
	"listbox", "menu", "menuseparator",
	"html:section", "html:div", "html:main", "html:span", "html:h2"
);
