/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";

var anonymox = {};
Components.utils.import("chrome://anonymox/content/modules/components.jsm", anonymox);

anonymox.panel = {

	lastOrigin : undefined,
	state : {
		active : false,
		site : undefined
	},


	open : function(origin, event) {
		let info = new anonymox.Info("panel.open");
		try {
			anonymox.ui.panelWindow = window;
			let panelElement = document.getElementById("anonymox-panel");

			if (!origin)
				origin = anonymox.panel.lastOrigin;

			if (!anonymox.state.isReady) {
				if (anonymox.state.isStarting) {
					if (origin == "networkStatus")
						return;
					else
						anonymox.ui.notification(1, "loadingPleaseWait");
				} else {
					anonymox.core.switchOn();
				}
				return;
			}

			const profile = anonymox.attachedData.get("profile", window);
			if (!profile) {
				let targetInfo;
				if (event) {
					targetInfo = event.target.id;
				}
				else {
					targetInfo = "not available";
				}

				info.error(
					"Profile not available in panel.open. Origin: " +origin+
					"; Target: " +targetInfo+
					"; System state: "
					+info.serializeObject(anonymox.state.systemState)+
					"; Off due to failure: " +anonymox.state.offDueToFailure
				);
			}



			info.sendMsg(anonymox.utils.msgType.usage, "panel", "open|" + origin);

			try {
				if ( origin == "viewButton" || origin == "mainMenu" || origin == "toolsMenu" || origin == "networkStatus") {
					panelElement.openPopup(document.getElementById('anonymox-view-button'), 'after_start', 0, 0, false, false);
				} else if (origin == "statusBar") {
					panelElement.openPopup(document.getElementById('anonymox-statusBarEntry'), 'before_start', 0, 0, false, false);
				} else {
					info.error("Unknown origin: '" + origin + "'");
				}
			} catch(e) {
				info.error(e, "origin = " + origin);
			}
			anonymox.panel.lastOrigin = origin;
		} catch(e) {
			info.error(e);
		}
	},

	profileChangedHook : function() {
		let panelElement = document.getElementById("anonymox-panel");

		if (panelElement.state == "open") {
			anonymox.panel.rebuildAll();
		}
	},

	onPopupShowingHandler : function() {
		let info = new anonymox.Info("panel.onPopupShowingHandler");

		let panelElement = document.getElementById("anonymox-panel");

		if (panelElement.state != "open") {
			anonymox.panel.rebuildAll();
		}
	},

	rebuildSiteSettings : function() {
		let info = new anonymox.Info("panel.rebuildSiteSettings");
		let jQ = window.anonymox.jQuery;

		const profile = anonymox.attachedData.get("profile", window);

		let main = jQ("#anonymox-panel-main");
		let areaSettings = main.find(".main-settings .area-settings");
		let areaSettingsDesc = areaSettings.find(".desc");

		if (!window.anonymoxdata.uri) {
			info.error(new Error("rebuildSiteSettings: window.anonymoxdata.uri is not set!"));
		}

		main.removeClass("status-internal");


		if (window.anonymoxdata.profile.isInternal) {
			anonymox.panel.hideIdentitiesList();
			main.removeClass("settings-saved").addClass("settings-empty");
			anonymox.panel.state.site = "internal";

			areaSettingsDesc.text("non-proxied page");
			main.addClass("status-internal");
			return;
		} else {
			main.removeClass("status-internal");
		}


		if (profile.site != "*") {
			main.removeClass("settings-empty").addClass("settings-saved");

			let filters = anonymox.utils.getDomainPartsFromURI(window.anonymoxdata.uri);
			if (filters != null) {
				let domain = filters[filters.length - 2];
				let readableDomain = anonymox.profilemgr.getReadableName(domain);
				areaSettingsDesc.html(anonymox.localize("panel.savedSettings", "<strong>" + readableDomain + "</strong>"));
				anonymox.panel.state.site = readableDomain;
			} else {
				anonymox.panel.state.site = "unknown";
			}
		} else {
			main.removeClass("settings-saved").addClass("settings-empty");
			areaSettingsDesc.text(anonymox.localize("panel.saveSettings"));
			anonymox.panel.state.site = "*";
		}
	},

	rebuildAll : function() {
		anonymox.panel.rebuildProxyInfo();
		anonymox.panel.rebuildPremTrial();
		anonymox.panel.rebuildActiveCheckBoxAndContainers();
		anonymox.panel.rebuildConnectionOverview();
		anonymox.panel.rebuildSiteSettings();

		if (window.anonymoxdata.profile.gatewayFilter.enabled) {
			anonymox.panel.rebuildIdentitiesList();
			anonymox.panel.rebuildCountryList();
		}

		anonymox.panel.apply(true);
	},

	identClickFuncMaker : function(gw) {
		return function() {
			let info = new anonymox.Info("identClickFuncMaker");
			window.anonymoxdata.profile.wantGateway(gw);
			info.sendMsg(anonymox.utils.msgType.usage, "features", "selectGateway");
		};
	},

	IdentityList : function(wrapper) {
		let list = wrapper.children("ul");
		let jQ = window.anonymox.jQuery;
		let info = new anonymox.Info("panel.IdentityList");
		return {
			clear : function() {
				wrapper.empty();
				list = jQ("<ul></ul>");
				wrapper.append(list);
			},
			addEntry : function(name, ip, quality, premium, current, gateway) {
				let entry = jQ("<li></li>");
				if (premium) {
					entry.addClass("premium");
				}
				if (current) {
					entry.addClass("current");
				}

				let prem = jQ("<div class=\"icon star\"></div>");
				let main = jQ("<div class=\"main\"></div>");

				let nameDiv = jQ("<div class=\"name\"></div>");
				nameDiv.text(name);



				let qualiIcon = jQ("<span class=\"icon quality\"></span>");
				qualiIcon.addClass("quality-" + quality);
				nameDiv.append(qualiIcon);
				main.append(nameDiv);

				let ipDiv = jQ("<div class=\"ip\"></div>");
				ipDiv.text(ip);
				main.append(ipDiv);

				let selIcon = jQ("<div class=\"icon selection\"></div>");


				entry.append(prem).append(main).append(selIcon);

				entry.click(function() {
					window.anonymoxdata.profile.wantGateway(gateway);
					info.sendMsg(anonymox.utils.msgType.usage, "features", "selectGateway");
				});

				list.append(entry);
			}
		};
	},

	rebuildIdentitiesList : function(invalidFilter) {
		let info = new anonymox.Info("panel.rebuildIdentitiesList");
		let jQ = window.anonymox.jQuery;

		let identityListWrapper = jQ(".overlay-identities").find(".column.identities");
		let identityList = anonymox.panel.IdentityList(identityListWrapper);
		identityList.clear();

		if (invalidFilter) {
			jQ(".overlay-identities").find(".title.identities .count").text(0);
			return;
		}

		jQ(".overlay-identities").find(".title.identities .count").text(window.anonymoxdata.profile.gatewayRotateList.length);

		for ( let i = 0; i < window.anonymoxdata.profile.gatewayRotateList.length; i++ ) {
			let gw = window.anonymoxdata.profile.gatewayRotateList[i];

			let quality = anonymox.panel.score(gw.usageIndex, -50, 100);

			identityList.addEntry(gw.id, gw.ip, quality, gw.attributes.premium, gw == window.anonymoxdata.profile.gateway, gw);
		}
	},


	score: function(u, umin, umax) {
		if (u <= umin) {
			return 5;
		} else if (u >= umax) {
			return 1;
		}
		let diff = umin - umax;
		return Math.round(1 + 4 * (u - umax) / diff, 0);
	},

	rebuildActiveCheckBoxAndContainers : function() {
		let mainProxy = document.getElementById('anonymox-panel-main-proxy');

		let isOn = window.anonymoxdata.profile.gatewayFilter.enabled;


		let mainStatusSection = window.anonymox.jQuery("#anonymox-main-status");
		mainStatusSection.removeClass("show-menu");

		if (window.anonymoxdata.profile.isInternal) {
			window.anonymox.jQuery(mainProxy).hide();
			anonymox.panel.setActiveState(false);
		} else {
			anonymox.panel.state.active = isOn;
			anonymox.panel.setActiveState(isOn);

			if (isOn) {
				window.anonymox.jQuery(mainProxy).show();
			} else {
				window.anonymox.jQuery(mainProxy).hide();
			}
		}
	},

	CountryList : function(wrapper) {
		let list = wrapper.children("ul");
		let jQ = window.anonymox.jQuery;
		return {
			clear : function() {
				wrapper.empty();
				list = jQ("<ul></ul>");
				wrapper.append(list);
			},
			addEntry : function(country, current) {

				let countryClass, countryLabel;
				if (country == "automatic") {
					countryClass = country;
					countryLabel = anonymox.localize("panel.allCountries");
				} else if (country == "premium-more") {
					countryClass = country;
					countryLabel = anonymox.localize("panelMoreCountries");
				} else {
					countryClass = "country-" + country;

					countryLabel = anonymox.utils.translateGW(country);
				}
				let entry = jQ("<li></li>").addClass(countryClass);
				entry.attr('data-country', country);


				if (current) {
					entry.addClass("current");
				}


				let flag = jQ("<div class=\"flag\"></div>");
				let name = jQ("<div class=\"name\"></div>");
				name.text(countryLabel);
				let selIcon = jQ("<div class=\"icon selection\"></div>");


				entry.append(flag).append(name).append(selIcon);
				list.append(entry);
			}
		}
	},

	rebuildCountryList : function () {
		let info = new anonymox.Info("panel.rebuildCountryList");

		let jQ = window.anonymox.jQuery;

		let countries = {};

		let countryListWrapper = jQ(".overlay-identities").find(".column.countries");
		let cL = anonymox.panel.CountryList(countryListWrapper);
		cL.clear();


		let wap = window.anonymoxdata.profile;
		let wapGwFilterIsDefined = wap != undefined && wap.gatewayFilter != undefined;
		let noCountryFilter = wapGwFilterIsDefined && wap.gatewayFilter.country == undefined;
		cL.addEntry("automatic", noCountryFilter);


		for (let gwi = 0; gwi < anonymox.core.gatewayListCache.length; gwi++) {
			let gw = anonymox.core.gatewayListCache[gwi];


			if (!countries[gw.country]) {
				countries[gw.country] = true;

				let selected = wapGwFilterIsDefined && wap.gatewayFilter.country == gw.country;
				cL.addEntry(gw.country, selected);
			}
		}

		if (anonymox.ui.showMoreCountries) {
			cL.addEntry("premium-more", false);
		}
	},

	activeButtonClick : function() {
		let info = new anonymox.Info("panel.activeButtonClick");

		if (window.anonymoxdata.profile.isInternal) {
			return;
		}


		anonymox.panel.state.active = !anonymox.panel.state.active;
		let active = anonymox.panel.state.active;

		let site = anonymox.panel.state.site;

		let curFilter = anonymoxdata.profile.gatewayFilter;
		if (!curFilter.enabled && active) {

			curFilter.enabled = true;
			anonymox.panel.setActiveState(true);
			anonymox.profilemgr.setProfile(curFilter, site);
			anonymox.panel.rebuildAll();
		} else if (curFilter.enabled && !active) {

			curFilter.enabled = false;
			anonymox.panel.setActiveState(false);
			anonymox.profilemgr.setProfile(curFilter, site);
			anonymox.panel.rebuildAll();
		}
	},

	setActiveState : function(active) {
		let main = window.anonymox.jQuery('#anonymox-panel-main');
		let activeToggleDesc = window.anonymox.jQuery(".area-active-toggle .desc");

		if (active) {
			main.removeClass("status-inactive").addClass("status-active");
			activeToggleDesc.text(anonymox.localize("panel.active"));
		} else {
			main.removeClass("status-active").addClass("status-inactive");
			activeToggleDesc.text(anonymox.localize("panel.inactive"));
		}
	},

	apply : function(dryRun) {
		let info = new anonymox.Info("panel.apply");
		try {
			let jQ = window.anonymox.jQuery;
			let site = anonymox.panel.state.site;

			let filter = {
				enabled : anonymox.panel.state.active,
				attributes : {}
			};

			if (filter.enabled) {

				let selectedCountry = anonymox.panel.getCurrentCountry();
				filter.country = selectedCountry != "automatic" ? selectedCountry : undefined;
			}


			if (anonymox.profilemgr.filterYieldsResults(filter, site)) {
				if (!dryRun) {

					anonymox.profilemgr.setProfile(filter, site);

					anonymox.panel.rebuildAll();

					info.sendMsg(anonymox.utils.msgType.usage, "panel", "apply|" + site + "|" + filter.enabled + "|" + anonymox.profilemgr.getCountryName(filter.country));
				}
			} else {
				info.notice("Current filter is invalid");


				if (!window.anonymoxdata.profile.gatewayFilter.enabled && filter.enabled) {
					info.notice("invalid filter, but want to start -> start with standard");
					anonymox.profilemgr.setProfile(anonymox.profilemgr.defaultFilter, site);
					anonymox.panel.rebuildActiveCheckBoxAndContainers();
				}

				anonymox.panel.rebuildIdentitiesList(true);
			}
		} catch(e) {
			info.error(e);
		}
	},


	rebuildProxyInfo : function() {
		let info = new anonymox.Info("panel.rebuildProxyInfo");

		let mainProxy = window.anonymox.jQuery("#anonymox-panel-main-proxy");
		let proxyInfo = mainProxy.children(".area-proxy-info");
		let proxyDesc = proxyInfo.find(".proxy-desc");
		let qualityInfo = proxyInfo.find(".icon.quality");

		proxyInfo.removeClass(function (idx, className) {
			return (className.match(/(^|\s)country-\S+/g) || []).join(" ");
		});

		let currentGw = window.anonymoxdata.profile.gateway;

		let countryClass = "country-" + currentGw.country;
		proxyInfo.addClass(countryClass);


		proxyDesc.text(currentGw.id + ", " + anonymox.utils.translateGW(currentGw.country));


		qualityInfo.removeClass(function (index, className) {
			return (className.match (/(^|\s)quality-\S+/g) || []).join(' ');
		});
		qualityInfo.addClass("quality-" + anonymox.panel.score(currentGw.usageIndex, -50, 100));
	},

	rebuildPremTrial : function() {
		let info = new anonymox.Info("panel.rebuildPremTrial");
		let premTrialButton = window.anonymox.jQuery(".area-premium-trial");

		if (!anonymox.core.isPremiumAccount && anonymox.core.showPremTrial) {
			premTrialButton.show();
		} else {
			premTrialButton.hide();
		}
	},

	rebuildConnectionOverview : function() {
		let info = new anonymox.Info("panel.rebuildConnectionOverview");

		window.anonymoxdata.uri = anonymox.attachedData.get("uri", window);

		let connOverview = window.anonymox.jQuery("#anonymox-panel-main-connection-overview");
		let connGraph = connOverview.find(".area-connection-graph");
		let connLP = connOverview.find(".connection-local-proxy");
		let connPT = connOverview.find(".connection-proxy-target");
		let connLT = connOverview.find(".connection-local-target");
		let connTarget = connPT.add(connLT);

		let currentGw = window.anonymoxdata.profile.gateway;

		let endToEndEncrypted = window.anonymoxdata.uri.startsWith("https://");

		let connLPClasses = [
			"not-secured-premium",
			"secured-premium",
			"not-secured-no-premium",
			"secured-no-premium"
		];


		let connLPAddClassIdx = currentGw.tls ? 0 : 2;

		if (endToEndEncrypted) {

			connLPAddClassIdx += 1;

			connTarget.removeClass("not-secured").addClass("secured");
		} else {
			connTarget.removeClass("secured").addClass("not-secured");
		}


		let connLPAddClass = connLPClasses.splice(connLPAddClassIdx, 1)[0];
		connLP.addClass(connLPAddClass).removeClass(connLPClasses.join(" "));

		let labelProxy = connOverview.find(".label-proxy");
		let nodeProxy = connOverview.find(".node-proxy");


		let labelTarget = connOverview.find(".label-target");
		let filters = anonymox.utils.getDomainPartsFromURI(window.anonymoxdata.uri);
		let siteName;
		if (filters != null) {
			let domain = filters[filters.length - 2];
			siteName = anonymox.profilemgr.getReadableName(domain);
		} else {
			siteName = window.anonymoxdata.uri;
		}
		labelTarget.find(".main .name").text(siteName);


		nodeProxy.removeClass(function (idx, className) {
			return (className.match(/(^|\s)country-\S+/g) || []).join(" ");
		});

		connOverview.find(".label-local").find(".ip").text(anonymox.profilemgr.nullGateway.externalInfo.ip);

		let targetIp = connOverview.find(".label-target").find(".ip");
		targetIp.text("-");
		if (filters != null) {
			anonymox.core.resolveDomain(filters[0], function(ip) {
				targetIp.text(ip);
			});
		}


		if (currentGw.nullGw) {
			connGraph.removeClass("local-proxy-target");
			connGraph.addClass("local-target");

			labelProxy.find(".ip").text("none");

			anonymox.panel.hideIdentitiesList();

			connLP.hide();
			connPT.hide();
			connLT.show();
			nodeProxy.hide();
			labelProxy.hide();
		} else {
			connGraph.removeClass("local-target");
			connGraph.addClass("local-proxy-target");
			let countryClass = "country-" + currentGw.country;
			nodeProxy.addClass(countryClass);

			labelProxy.find(".node-name").text(currentGw.id);
			if (currentGw.externalInfo) {
				labelProxy.find(".ip").text(currentGw.externalInfo.ip);
			}

			connLP.show();
			connPT.show();
			connLT.hide();
			nodeProxy.show();
			labelProxy.show();
		}
	},

	setCurrentCountry: function(country, target) {
		let info = new anonymox.Info("setCurrentCountry");
		let jQ = window.anonymox.jQuery;
		let countryListWrapper = jQ(".overlay-identities").find(".column.countries");
		countryListWrapper.find("li").removeClass("current");
		jQ(target).addClass("current");
	},

	getCurrentCountry: function() {
		let info = new anonymox.Info("getCurrentCountry");
		let jQ = window.anonymox.jQuery;
		let current = jQ(".overlay-identities").find(".column.countries").find(".current");
		let country = current.attr('data-country');
		return country;
	},

	showIdentitiesList: function() {
		let identitiesOverlay = window.anonymox.jQuery(".overlay-identities");
		identitiesOverlay.show();
	},

	hideIdentitiesList: function() {
		let identitiesOverlay = window.anonymox.jQuery(".overlay-identities");
		identitiesOverlay.hide();
	},

	toggleDetails: function() {
		let info = new anonymox.Info("toggleDetails");
		let connOverview = window.anonymox.jQuery(".main-connection-overview");
		connOverview.toggleClass("show-details");
	},



	countryListCmd : function(el) {
		let info = new anonymox.Info("countryListCmd");
		let country = el.getAttribute('data-country');

		if (country == "premium-more") {
			anonymox.browserapi.openURI(anonymox.config.premiumInfoURI + "morecountries", 0);
			anonymox.panel.closePanel();
		} else {
			anonymox.panel.setCurrentCountry(country, el);
			anonymox.panel.apply();
		}
	},

	siteSettingsCmd : function() {

		let info = new anonymox.Info("panel.siteSettingsCmd");

		if (window.anonymoxdata.profile.isInternal) {
			return;
		}

		let jQ = window.anonymox.jQuery;
		let main = jQ("#anonymox-panel-main");
		let areaSettings = main.find(".main-settings .area-settings");
		let areaSettingsDesc = areaSettings.find(".desc");
		let profile = anonymox.panel.state.site;

		if (profile == "*") {


			main.removeClass("settings-empty").addClass("settings-saved");

			let filters = anonymox.utils.getDomainPartsFromURI(window.anonymoxdata.uri);
			if (filters != null) {
				let domain = filters[filters.length - 2];
				let readableDomain = anonymox.profilemgr.getReadableName(domain);

				areaSettingsDesc.html(anonymox.localize("panel.savedSettings", "<strong>" + readableDomain + "</strong>"));
				anonymox.panel.state.site = domain;

				anonymox.panel.apply();
			}
		} else {

			let promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", anonymox.localize("changeProfileDeleteHint", anonymox.profilemgr.getReadableName(window.anonymoxdata.profile.site)))) {
				window.anonymoxdata.profile.removeFromProfileList();
				anonymox.panel.rebuildSiteSettings();
			}
			anonymox.panel.open();
			return;
		}
	},

	closePanel : function() {
		document.getElementById('anonymox-panel').hidePopup();
	},

	menu : function() {
		let mainMenu = document.getElementById("anonymox-mainMenu");
		mainMenu.openPopup(document.getElementById('anonymox-panel-menubutton'), 'after_start', 0, 0, false, false);
	},

	showPreferences : function() {
		let info = new anonymox.Info("showPreferences");
		let dialog = openDialog("chrome://anonymox/content/prefs.xul", "anonymox-prefs", "centerscreen, chrome, toolbar");
		anonymox.panel.prefDialog = dialog;
	},



	toggleCookieActionPicker : function() {
		let info = new anonymox.Info("toggleCookieActionPicker");
		let jQ = window.anonymox.jQuery;

		let mainStatusSection = jQ("#anonymox-main-status");
		let cookieAction = window.anonymoxdata.profile.cookieAction;

		mainStatusSection.children(".area-menu").find(".entry").each(function (idx) {
			if (idx == cookieAction) {
				jQ(this).addClass("current");
			} else {
				jQ(this).removeClass("current");
			}
		});
		mainStatusSection.toggleClass("show-menu");
	}

};
