/*
* anonymoX client
*   for Firefox
*
* Author 	Nils Hügelmann	<nils.h@anonymox.net>
* Author 	Florian Prott	<florian.p@anonymox.net>
* Author 	Christian Otto	<chris.o@anonymox.net>
* Author 	Hanno Braun	<hanno.b@anonymox.net>
*
* www.anonymox.net
*
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
*
* Use at your own risk
*
* This is the released, uncommented version of the code.
*/

"use strict";

var anonymox = {};

Components.utils.import("chrome://anonymox/content/modules/components.jsm", anonymox);

function toggleUsageCollection() {
	anonymox.prefs.setIntPref('usagecollection', document.getElementById("anonymox-usageCollectionBox").checked ? 1 : 0 );
}

window.addEventListener("load", onLoadListener, false);
function onLoadListener(e) {
	window.removeEventListener("load", onLoadListener);


	document.getElementById("anonymox-usageCollectionBox").checked = anonymox.prefs.getIntPref("usagecollection", 0) == 1;

	updatePremiumStatus();
	updateCookieStatus();
	updateUserAgentStatus();
	updateReferrerStatus();

	populateNotificationSettingsList();
}

function activatePremium() {
	anonymox.core.activatePremium(document.getElementById("anonymox-activatePremium_code").value, updatePremiumStatus);
}

function updatePremiumStatus() {
	document.getElementById("anonymox-activatePremiumCodeBox1").setAttribute("hidden", (true == anonymox.core.isPremiumAccount));
	document.getElementById("anonymox-activatePremiumCodeBox2").setAttribute("hidden", !(true == anonymox.core.isPremiumAccount));
}

function updateCookieStatus() {
	let info = new anonymox.Info("prefs.xul.updateCookieStatus");

	let targetId = "anonymox-prefPanel-clearcookies-" + anonymox.config.allPrefs.getIntPref("network.cookie.cookieBehavior");
	let selectedItem = document.getElementById(targetId);

	let cookiesRadioGroup = document.getElementById("anonymox-prefPanel-clearcookies");
	cookiesRadioGroup.selectedItem = selectedItem;
}

function updateUserAgentStatus() {
	let info = new anonymox.Info("prefs.xul.updateUserAgentStatus");
	let active;
	try {
		active = anonymox.config.allPrefs.getCharPref("general.useragent.override");
	} catch (e) {

		active = "[[DEFAULT]]";
	}
	let userAgentRadioGroup = document.getElementById("anonymox-prefPanel-useragent");
	for ( let i = 0; i < userAgentRadioGroup.childNodes.length; i++) {
		const element = userAgentRadioGroup.childNodes[i];
		const value   = element.getAttribute("value");
		if (value == active || value == "[[CUSTOM]]") {
			userAgentRadioGroup.selectedItem = element;
			break;
		}
	}
}

function updateReferrerStatus() {
	document.getElementById("anonymox-prefPanel-toggleReferrer").setAttribute("checked",
			anonymox.config.allPrefs.getIntPref("network.http.sendRefererHeader") == 0);
}

function populateNotificationSettingsList() {
	let info = new anonymox.Info("prefs.xul.populateNotificationSettingsList");

	let notificationPreferences = document.getElementById("anonymox-notificationPreferences");
	let notificationList = document.getElementById("anonymox-notificationList");

	for ( let i = 0, pref; pref = notificationPreferences.children[i]; i++ ) {
		let prefShortName = pref.name.replace("extensions.anonymox.showNotifications.", "");
		let prefLocalPath = "showNotifications." + prefShortName;
		try {
			let checkBox = document.createElement("checkbox");
			checkBox.setAttribute("preference", pref.id);
			checkBox.setAttribute("label", anonymox.utils.localize(prefShortName + ".pref"));
			checkBox.setAttribute("checked", anonymox.prefs.getBoolPref(prefLocalPath));
			notificationList.appendChild(checkBox);
		} catch(e) {
			info.debug(prefLocalPath);
		}
	}
}

function usageCollectionInfo() {
	anonymox.browserapi.openURI(anonymox.config.usageCollectionInfoUri, 0);
}
